/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include <kglobal.h>
#include <qvariant.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlabel.h>
#include <kprogress.h>
#include <qbuttongroup.h>
#include <kicondialog.h>
#include <ktextedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qeventloop.h>
#include <qapplication.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <qtimer.h>

#include "kat.h"
#include "dlgcreatecatalog.h"
#include "dlgmainwindow.h"
#include "katcatalog.h"
#include "katscanfolder.h"
#include "katinfoextractor.h"
#include "kat_util.h"

DlgCreateCatalog::DlgCreateCatalog( DlgMainWindow* parent, const char* name, bool modal )
    : KDialogBase( parent, name, modal, i18n( "Creation of the catalog" ) , KDialogBase::Ok|KDialogBase::Cancel|KDialogBase::Help, KDialogBase::Ok, true )
{
    mf = parent;
    m_catalog = mf->catalog();

    QWidget *page = new QWidget( this );
    setMainWidget(page);

    QVBoxLayout *DlgCreateCatalogLayout = new QVBoxLayout( page, 11, 6, "DlgCreateCatalogLayout" );

    tabWidget = new QTabWidget( page, "tabWidget" );

    QWidget *tStatus = new QWidget( tabWidget, "tStatus" );

    QVBoxLayout *layout1 = new QVBoxLayout( tStatus, KDialog::marginHint(), KDialog::spacingHint(), "layout1" );

    tlMessage = new QLabel( tStatus, "tlMessage" );
    tlMessage->setAlignment( int( QLabel::AlignCenter ) );
    tlMessage->setText( i18n( "The specified folder is being scanned.\nPlease wait." ) );
    layout1->addWidget( tlMessage );

    pProgress = new KProgress( tStatus, "pProgress" );
    pProgress->setPercentageVisible( false );
    pProgress->setTotalSteps( 0 );
    pProgress->setProgress( 0 );
    pProgress->setTextEnabled( false );
    pProgress->setFormat( QString::null );
    layout1->addWidget( pProgress );

    tlTimeFromStart = new QLabel( tStatus, "tlTimeFromStart" );
    tlTimeFromStart->setAlignment( int( QLabel::AlignCenter ) );
    tlTimeFromStart->setText( QString::null );
    layout1->addWidget( tlTimeFromStart );

    bgStats = new QButtonGroup( tStatus, "bgStats" );
    bgStats->setTitle( i18n( "Analyzed" ) );
    layout1->addWidget( bgStats );

    QVBoxLayout *layout3=new QVBoxLayout( bgStats, KDialog::marginHint(), KDialog::spacingHint(), "layout3" );

    QGridLayout *grid1 = new QGridLayout( bgStats, 2, 3 , KDialog::marginHint(), KDialog::spacingHint());

    tlFiles = new QLabel( bgStats, "tlFiles" );
    tlFiles->setText( i18n( "Files" ) );
    grid1->addWidget( tlFiles,0,0 );


    tlFilesNum = new QLabel( bgStats, "tlFilesNum" );
    //tlFilesNum->setAlignment( Qt::AlignRight );
    tlFilesNum->setText( QString::null );
    grid1->addWidget( tlFilesNum,1,0 );

    tlFolders = new QLabel( bgStats, "tlFolders" );
    tlFolders->setText( i18n( "Folders" ) );
    grid1->addWidget( tlFolders,0,1 );

    tlFoldersNum = new QLabel( bgStats, "tlFoldersNum" );
    //tlFoldersNum->setAlignment( Qt::AlignRight );
    tlFoldersNum->setText( QString::null );
    grid1->addWidget( tlFoldersNum,1,1 );

    tlFileSize = new QLabel( bgStats, "tlFileSize" );
    tlFileSize->setText( i18n( "File size" ) );
    grid1->addWidget( tlFileSize,0,2 );


    tlFileSizeNum = new QLabel( bgStats, "tlFileSizeNum" );
    //tlFileSizeNum->setAlignment( Qt::AlignRight );
    tlFileSizeNum->setText( QString::null );
    grid1->addWidget( tlFileSizeNum,1,2 );
    layout3->addLayout( grid1 );


    grid1 = new QGridLayout( bgStats, 4, 2 );


    tlWords = new QLabel( bgStats, "tlWords" );
    tlWords->setText( i18n( "Words imported:" ) );
    grid1->addWidget( tlWords, 0, 0 );

    tlWordsNum = new QLabel( bgStats, "tlWordsNum" );
    //tlWordsNum->setAlignment( Qt::AlignRight );
    tlWordsNum->setText( QString::null );
    grid1->addWidget( tlWordsNum, 0, 1 );

    tlMetaData = new QLabel( bgStats, "tlMetaData" );
    tlMetaData->setText( i18n( "Metadata imported:" ) );
    grid1->addWidget( tlMetaData, 1, 0 );

    tlMetaDataNum = new QLabel( bgStats, "tlMetaDataNum" );
    //tlMetaDataNum->setAlignment( QLabel::AlignRight );
    tlMetaDataNum->setText( QString::null );
    grid1->addWidget( tlMetaDataNum, 1, 1 );

    tlFullTexts = new QLabel( bgStats, "tlFullTexts" );
    tlFullTexts->setText( i18n( "Fulltexts imported:" ) );
    grid1->addWidget( tlFullTexts, 2, 0 );

    tlFullTextsNum = new QLabel( bgStats, "tlFullTextsNum" );
    //tlFullTextsNum->setAlignment( QLabel::AlignRight );
    tlFullTextsNum->setText( QString::null );
    grid1->addWidget( tlFullTextsNum, 2, 1 );

    tlThumbnails = new QLabel( bgStats, "tlThumbnails" );
    tlThumbnails->setText( i18n( "Thumbnails imported:" ) );
    grid1->addWidget( tlThumbnails, 3, 0 );

    tlThumbnailsNum = new QLabel( bgStats, "tlThumbnailsNum" );
    //tlThumbnailsNum->setAlignment( QLabel::AlignRight );
    tlThumbnailsNum->setText( QString::null );
    grid1->addWidget( tlThumbnailsNum, 3, 1 );
    layout3->addLayout( grid1 );

    tabWidget->insertTab( tStatus, i18n( "Status" ) );

    QWidget *tDetails = new QWidget( tabWidget, "tDetails" );
    QVBoxLayout *layout2 = new QVBoxLayout( tDetails, KDialog::marginHint(), KDialog::spacingHint(), "layout2" );

    teDetails = new KTextEdit( tDetails, "teDetails" );
    QFont teDetails_font(  teDetails->font() );
    //TODO use font from KDE
    teDetails_font.setFamily( "Monospace" );
    teDetails->setFont( teDetails_font );
    tabWidget->insertTab( tDetails, i18n( "Details" ) );
    layout2->addWidget( teDetails );

    DlgCreateCatalogLayout->addWidget( tabWidget );

    resize( QSize(560, 460).expandedTo( minimumSizeHint() ) );
    clearWState( WState_Polished );


    // timers
    QTimer::singleShot(0, this, SLOT( startScan() ) );

    updateTimer = new QTimer( this );
    updateTimer->start( 300 );
    connect( updateTimer, SIGNAL( timeout() ), SLOT( updateScanDialog() ) );
}

DlgCreateCatalog::~DlgCreateCatalog() {
}

void DlgCreateCatalog::startScan() {
    m_start.start();
    enableButtonOK( false );

    mf->databaseManager()->createKatDatabase( QDir::currentDirPath() + "/temp.kat" );

    m_ksf = KatScanFolder::scanFolderJob( KURL( m_catalog->getPath() ), m_catalog, mf->databaseManager() );
    connect( m_ksf, SIGNAL( completed() ), this, SLOT( scanCompleted() ) );
    connect( this, SIGNAL( aborted() ), m_ksf, SLOT( slotScanAborted() ) );
}

void DlgCreateCatalog::slotCancel()
{
    kdDebug() << "abort emitted" << endl;
    emit aborted();
    done( QDialog::Rejected );
}

void DlgCreateCatalog::updateScanDialog() {
    pProgress->setProgress( pProgress->progress() + 1 );

    int etime = m_start.elapsed();
    int seconds = etime/1000;

    QString folders = QString::number( m_catalog->getFolders() );
    addSeparators( folders );

    QString files = QString::number( m_catalog->getFiles() );
    addSeparators( files );

    QString fileSize = QString::number( m_catalog->getFileSize() );
    addSeparators( fileSize );

    tlTimeFromStart->setText( convertSeconds( seconds ) );

    tlFoldersNum->setText( folders );
    tlFilesNum->setText( files );
    tlFileSizeNum->setText( fileSize );
}

void DlgCreateCatalog::updateExtractDialog() {
    pProgress->setProgress( m_catalog->getExtractions() );

    int etime = m_start.elapsed();
    int seconds = etime/1000;

    QString metadata = QString::number( m_catalog->getMetaData() );
    addSeparators( metadata );

    QString thumbnails = QString::number( m_catalog->getThumbnails() );
    addSeparators( thumbnails );

    QString fulltexts = QString::number( m_catalog->getFullTexts() );
    addSeparators( fulltexts );

    QString words = QString::number( m_catalog->getWords() );
    addSeparators( words );

    tlTimeFromStart->setText( convertSeconds( seconds ) );

    tlMetaDataNum->setText( metadata );
    tlThumbnailsNum->setText( thumbnails );
    tlFullTextsNum->setText( fulltexts );
    tlWordsNum->setText( words );
}

void DlgCreateCatalog::scanCompleted() {
    // if the scan has been too quick to display the stats,
    // display them now (it happens in presence of few files)
    updateScanDialog();

    // disconnect ksf from SIGNAL aborted()
    disconnect( this, SIGNAL( aborted() ) );

    updateTimer->stop();

    updateTimer = new QTimer( this );
    updateTimer->start( 300 );
    connect( updateTimer, SIGNAL( timeout() ), SLOT( updateExtractDialog() ) );

    pProgress->setTotalSteps( m_catalog->getFiles() * 3 ); // *3 = previews, metadata, fulltexts
    pProgress->setProgress( 0L );
    pProgress->setTextEnabled( true );
    pProgress->setFormat( "%p%" );

    m_kie = KatInfoExtractor::extractInfoJob( m_catalog, mf->databaseManager() );
    connect( m_kie, SIGNAL( completed() ), this, SLOT( extractionCompleted() ) );
    connect( this, SIGNAL( aborted() ), m_kie, SLOT( slotExtractionAborted() ) );
}

void DlgCreateCatalog::extractionCompleted() {
    updateTimer->stop();
    pProgress->setProgress( m_catalog->getFiles() * 3 );

    m_catalog->saveCatalogInfo(mf->databaseManager());

    enableButtonOK( true );
    KMessageBox::information( this, i18n( "Done!" ) );
}

#include "dlgcreatecatalog.moc"
