/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kglobal.h>
#include <kstandarddirs.h>
#include <ctime>
#include <qapplication.h>
#include <klocale.h>
#include <kkeydialog.h>
#include <kmessagebox.h>
#include <krun.h>
#include <kedittoolbar.h>
#include <kmimetype.h>
#include <qfiledialog.h>
#include <qvaluevector.h>
#include <ktrader.h>
#include <kuserprofile.h>

#include "kat.h"
#include "dlgmainwindow.h"
#include "dlgnewcatalog.h"
#include "dlgcreatecatalog.h"
#include "dlgsearch.h"
#include "katiconviewitem.h"
#include "katlistviewitem.h"
#include "dlgshowfulltext.h"
#include "dlgshowwordslist.h"
#include "katlanguagemanager.h"
#include "dlglanguagemanagement.h"
#include "kat_util.h"
#include "katlistview.h"
#include "katiconview.h"

#define FILE_NAME 0
#define FILE_EXTENSION 1
#define FILE_MIMETYPE 2
#define FILE_SIZE 3
#define FILE_DATE 4
#define FILE_TIME 5
#define FILE_THUMBNAIL 6
#define FILE_METADATA 7
#define FILE_FULLTEXT 8
#define FILE_LANGUAGE 9

#define CATALOG_NAME 0
#define CATALOG_BASEPATH 1
#define CATALOG_AUTHOR 2
#define CATALOG_CREATIONDATE 3
#define CATALOG_LASTUPDATE 4
#define CATALOG_FILES 5
#define CATALOG_WORDS 6

DlgMainWindow::DlgMainWindow() : DockMainWindow ( 0, "KAT" ) {
    prevFileType = QString::null;
    dm = new KatDatabaseManager(this);
    // load all language profiles
    KatLanguageManager::loadAllLanguageProfiles(lp);

    setCaption( i18n( "Kat" ) );
    m_config = kapp->config();

    m_kat = 0L;

    initView();
    initStatusBar();
    initActions();
    createGUI( 0L );
}

DlgMainWindow::~DlgMainWindow() {
    kdDebug()<<" DlgMainWindow::~DlgMainWindow()****************************\n";
    slotSaveProperties();
    lvFiles->saveLayout( m_config, "Header-Order" );
    delete dm;
    delete m_kat;
}

void DlgMainWindow::setCatalog(KatCatalog*_kat)
{
    delete m_kat;
    m_kat = _kat;
}

void DlgMainWindow::initActions() {
    fileNewAction = KStdAction::openNew( this, SLOT( slotFileNew() ), actionCollection() );
    fileNewAction->setToolTip( i18n( "Create a new catalog" ) );

    fileOpenAction = KStdAction::open( this, SLOT( slotFileOpen() ), actionCollection() );
    fileOpenAction->setToolTip( i18n( "Open a previously saved catalog" ) );

    fileSaveAction = KStdAction::save( this, SLOT( slotFileSave() ), actionCollection() );
    fileSaveAction->setToolTip( i18n( "Save current catalog" ) );

    fileOpenRecent = KStdAction::openRecent( this, SLOT( slotOpenURL( const KURL& ) ), actionCollection() );
    fileOpenRecent->loadEntries( m_config );

    fileSaveAsAction = KStdAction::saveAs( this, SLOT( slotFileSaveAs() ), actionCollection() );
    fileSaveAsAction->setToolTip( i18n( "Save current catalog with a different file name" ) );

    filePrintAction = KStdAction::print( this, SLOT( slotFilePrint() ), actionCollection() );
    fileExitAction = KStdAction::quit( this, SLOT( close() ), actionCollection() );
    editUndoAction = KStdAction::undo( this, SLOT( slotEditUndo() ), actionCollection() );
    editRedoAction = KStdAction::redo( this, SLOT( slotEditRedo() ), actionCollection() );
    editCutAction = KStdAction::cut( this, SLOT( slotEditCut() ), actionCollection() );
    editCopyAction = KStdAction::copy( this, SLOT( slotEditCopy() ), actionCollection() );
    editPasteAction = KStdAction::paste( this, SLOT( slotEditPaste() ), actionCollection() );
    editFindAction = KStdAction::find( this, SLOT( slotEditFind() ), actionCollection() );

    (void)KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );

    KStdAction::configureToolbars( this, SLOT( slotEditToolbars() ), actionCollection() );

    toolbarSearch = new KToggleAction( i18n( "Search Files" ), "filefind", 0, this, SLOT( slotFileFind() ),
                                 actionCollection(), "toolbar_search_files");
    toolbarSearch->setToolTip( i18n( "Find files in the catalog" ) );

    toolbarViewDetailed = new KToggleAction( i18n( "Show Detailed View" ), "view_detailed", 0, this,
                                 SLOT(slotViewDetailed() ),
                                 actionCollection(), "toolbar_view_detailled" );
    toolbarViewDetailed->setToolTip( i18n( "Detailed view" ) );
    toolbarViewDetailed->setExclusiveGroup( "viewmodes" );
    toolbarViewDetailed->setEnabled( false );
    toolbarViewDetailed->setChecked( true );

    toolbarViewIcons = new KToggleAction( i18n( "Show Detailed View" ), "view_icon", 0, this, SLOT( slotViewIcons() ),
                                actionCollection(), "toolbar_view_icons" );
    toolbarViewIcons->setToolTip( i18n( "Icon view" ) );
    toolbarViewIcons->setExclusiveGroup( "viewmodes" );
    toolbarViewIcons->setEnabled( false );

    toolbarViewThumbnails = new KToggleAction(i18n( "Show Detailed View" ), "thumbnail", 0, this,
                                 SLOT(slotViewThumbnails()),
                                 actionCollection(), "toolbar_view_thumbnails" );
    toolbarViewThumbnails->setToolTip( i18n( "Thumbnails view" ) );
    toolbarViewThumbnails->setExclusiveGroup( "viewmodes" );
    toolbarViewThumbnails->setEnabled( false );

    toolbarConfigure= new KAction( i18n( "Configure Kat" ), "configure", 0, this, SLOT( slotConfigureKat() ),
                                actionCollection(), "toolbar_configure_kat" );

    toolbarConfigure->setToolTip( i18n("Configure Kat") );

    connect( lvFolders, SIGNAL( doubleClicked( QListViewItem* ) ),
             this, SLOT( doubleClickedFolder( QListViewItem* ) ) );
    connect( lvFolders, SIGNAL( currentChanged( QListViewItem* ) ),
             this, SLOT( selectFolder( QListViewItem* ) ) );
    connect( lvFolders, SIGNAL( expanded( QListViewItem* ) ),
             this, SLOT( expandFolder( QListViewItem* ) ) );

    connect( lvFiles, SIGNAL( contextMenu( KListView*, QListViewItem*, const QPoint& ) ),
             this, SLOT( contextFile( KListView*, QListViewItem*, const QPoint& ) ) );
    connect( lvFiles, SIGNAL( contextMenuRequested( QListViewItem*, const QPoint&,int ) ),
             this, SLOT( contextReqFile( QListViewItem*, const QPoint&, int ) ) );
    connect( lvFiles, SIGNAL( doubleClicked( QListViewItem* ) ),
             this, SLOT( doubleClickedFile( QListViewItem* ) ) );
    connect( lvFiles, SIGNAL( rightButtonClicked( QListViewItem*, const QPoint&, int ) ),
             this, SLOT( rightButtonFile( QListViewItem*, const QPoint&, int ) ) );
    connect( lvFiles, SIGNAL( currentChanged( QListViewItem* ) ),
             this, SLOT( selectFile( QListViewItem* ) ) );

    connect( lvResults, SIGNAL( contextMenu( KListView*, QListViewItem*, const QPoint& ) ),
             this, SLOT( contextFile( KListView*, QListViewItem*, const QPoint& ) ) );
    connect( lvResults, SIGNAL( contextMenuRequested( QListViewItem*, const QPoint&,int ) ),
             this, SLOT( contextReqFile( QListViewItem*, const QPoint&, int ) ) );
    connect( lvResults, SIGNAL( doubleClicked( QListViewItem* ) ),
             this, SLOT( doubleClickedFile( QListViewItem* ) ) );
    connect( lvResults, SIGNAL( rightButtonClicked( QListViewItem*, const QPoint&, int ) ),
             this, SLOT( rightButtonFile( QListViewItem*, const QPoint&, int ) ) );
    connect( lvResults, SIGNAL( currentChanged( QListViewItem* ) ),
             this, SLOT( selectFile( QListViewItem* ) ) );

    connect( ivFiles, SIGNAL( currentChanged( QIconViewItem* ) ),
             this, SLOT( selectFileIcon( QIconViewItem* ) ) );
    connect( ivFiles, SIGNAL( doubleClicked( QIconViewItem* ) ),
             this, SLOT( doubleClickedFileIcon( QIconViewItem* ) ) );
    connect( ivFiles, SIGNAL( contextMenuRequested( QIconViewItem*, const QPoint& ) ),
             this, SLOT( contextFileIcon( QIconViewItem*, const QPoint& ) ) );

    connect( ivResults, SIGNAL( currentChanged( QIconViewItem* ) ),
             this, SLOT( selectFileIcon( QIconViewItem* ) ) );
    connect( ivResults, SIGNAL( doubleClicked( QIconViewItem* ) ),
             this, SLOT( doubleClickedFileIcon( QIconViewItem* ) ) );
    connect( ivResults, SIGNAL( contextMenuRequested( QIconViewItem*, const QPoint& ) ),
             this, SLOT( contextFileIcon( QIconViewItem*, const QPoint& ) ) );

    connect( ivFilesThumbnails, SIGNAL( currentChanged( QIconViewItem* ) ),
             this, SLOT( selectFileThumbnail( QIconViewItem* ) ) );
    connect( ivFilesThumbnails, SIGNAL( doubleClicked( QIconViewItem* ) ),
             this, SLOT( doubleClickedFileThumbnail( QIconViewItem* ) ) );
    connect( ivFilesThumbnails, SIGNAL( contextMenuRequested( QIconViewItem*, const QPoint& ) ),
             this, SLOT( contextFileThumbnail( QIconViewItem*, const QPoint& ) ) );

    connect( ivResultsThumbnails, SIGNAL( currentChanged( QIconViewItem* ) ),
             this, SLOT( selectFileThumbnail( QIconViewItem* ) ) );
    connect( ivResultsThumbnails, SIGNAL( doubleClicked( QIconViewItem* ) ),
             this, SLOT( doubleClickedFileThumbnail( QIconViewItem* ) ) );
    connect( ivResultsThumbnails, SIGNAL( contextMenuRequested( QIconViewItem*, const QPoint& ) ),
             this, SLOT( contextFileThumbnail( QIconViewItem*, const QPoint& ) ) );

    disableAction();
}

void DlgMainWindow::disableAction() {
    filePrintAction->setEnabled( false );
    editUndoAction->setEnabled( false );
    editRedoAction->setEnabled( false );
    editCutAction->setEnabled( false );
    editCopyAction->setEnabled( false );
    editPasteAction->setEnabled( false );
    editFindAction->setEnabled( false );
    fileSaveAction->setEnabled( false );
    fileSaveAsAction->setEnabled( false );
}

void DlgMainWindow::slotOpenURL( const KURL&_url ) {
    fileOpen( _url );
}

void DlgMainWindow::initStatusBar() {
    //TODO
}

void DlgMainWindow::slotConfigureKat() {
    DlgLanguageManagement lm;
    lm.exec();
}

void DlgMainWindow::slotEditToolbars() {
    saveMainWindowSettings( m_config, "main_window_settings" );
    KEditToolbar dlg( factory(),this );
    connect( &dlg, SIGNAL( newToolbarConfig() ), SLOT( slotNewToolBarConfig() ) );
    dlg.exec();
}

void DlgMainWindow::slotNewToolBarConfig() {
  applyMainWindowSettings( m_config, "main_window_settings" );
}

void DlgMainWindow::initView() {

    //Find the Kat application data path
    QString appDataPath = locate( "data", "kat/kat_bg.png" );

    // File tree
    pmCatalogIcon = loadMenuIconEnabled( "kate" );
    pmDiscIcon = loadMenuIconEnabled( "hdd_unmount" );
    pmFolderIconSmall = loadMenuIconEnabled( "folder" );
    pmFileIconSmall = loadMenuIconEnabled( "unknown" );
    pmCheckIconSmall = loadMenuIconEnabled( "button_ok" );
    pmFolderIconLarge = loadLargeIconEnabled( "folder" );
    pmFileIconLarge = loadLargeIconEnabled( "unknown" );

    // Set up the status bar
    // Laurent: not used for the moment
    //sbStatusBar = statusBar();

    // DOCK MANAGER
    manager()->setSplitterHighResolution( true );
    manager()->setSplitterOpaqueResize( true );
    manager()->setSplitterKeepSize( false );

    // DOCK WIDGETS
    // Folders dock
    dockFolders = createDockWidget( "folders_tree", NULL, 0L, kapp->makeStdCaption( "Folders Tree" ),
                                    "Folders Tree" );
    lvFolders = new KListView( this );
    lvFolders->addColumn( i18n( "Catalogs" ) );
    lvFolders->setColumnAlignment( 0, Qt::AlignLeft );
    lvFolders->setRootIsDecorated( true );
    lvFolders->setSelectionMode( QListView::Single );
    lvFolders->header()->hide(); // TODO: ?????

    dockFolders->setWidget( lvFolders );

    // Files dock
    dockFiles = createDockWidget( "files_view", NULL, 0L,
                                  kapp->makeStdCaption( "Files view" ),
                                  "Files view" );
    lvFiles = new KatListView( this );
    lvFiles->addColumn( i18n( "Filename" ) );
    lvFiles->addColumn( i18n( "Ext" ) );
    lvFiles->addColumn( i18n( "Type" ) );
    lvFiles->addColumn( i18n( "Size" ) );
    lvFiles->addColumn( i18n( "Date" ) );
    lvFiles->addColumn( i18n( "Time" ) );
    lvFiles->addColumn( i18n( "Thumbnail" ) );
    lvFiles->addColumn( i18n( "Metadata" ) );
    lvFiles->addColumn( i18n( "Fulltext" ) );
    lvFiles->addColumn( i18n( "Language" ) );
    lvFiles->setSorting( FILE_NAME, true );
    lvFiles->setAllColumnsShowFocus( TRUE );
    lvFiles->setRootIsDecorated( FALSE );
    lvFiles->setShowSortIndicator( TRUE );
    lvFiles->setColumnAlignment( FILE_NAME, Qt::AlignLeft );
    lvFiles->setColumnAlignment( FILE_EXTENSION, Qt::AlignLeft );
    lvFiles->setColumnAlignment( FILE_MIMETYPE, Qt::AlignLeft );
    lvFiles->setColumnAlignment( FILE_SIZE, Qt::AlignRight );
    lvFiles->setColumnAlignment( FILE_DATE, Qt::AlignRight );
    lvFiles->setColumnAlignment( FILE_TIME, Qt::AlignRight );
    lvFiles->setColumnAlignment( FILE_THUMBNAIL, Qt::AlignCenter );
    lvFiles->setColumnAlignment( FILE_METADATA, Qt::AlignCenter );
    lvFiles->setColumnAlignment( FILE_FULLTEXT, Qt::AlignCenter );
    lvFiles->setColumnAlignment( FILE_LANGUAGE, Qt::AlignLeft );
    lvFiles->header()->setLabel( FILE_NAME, i18n( "Filename" ) );
    lvFiles->header()->setLabel( FILE_EXTENSION, i18n( "Ext" ) );
    lvFiles->header()->setLabel( FILE_MIMETYPE, i18n( "Type" ) );
    lvFiles->header()->setLabel( FILE_SIZE, i18n( "Size" ) );
    lvFiles->header()->setLabel( FILE_DATE, i18n( "Date" ) );
    lvFiles->header()->setLabel( FILE_TIME, i18n( "Time" ) );
    lvFiles->header()->setLabel( FILE_THUMBNAIL, i18n( "Thumbnail" ) );
    lvFiles->header()->setLabel( FILE_METADATA, i18n( "Metadata" ) );
    lvFiles->header()->setLabel( FILE_FULLTEXT, i18n( "Fulltext" ) );
    lvFiles->header()->setLabel( FILE_LANGUAGE, i18n( "Language" ) );
    lvFiles->setSorting( FILE_NAME, true );
    lvFiles->restoreLayout( m_config, "Header-Order" );
    lvFiles->setSelectionMode( QListView::Extended );

    filesMenu = new QPopupMenu( lvFiles );

    ivFiles = new KatIconView( this );
    ivFiles->hide();
    ivFiles->setResizeMode( QIconView::Adjust );
    ivFiles->setItemTextPos( QIconView::Bottom );
    ivFiles->setGridX( 80 );
    ivFiles->setGridY( 80 );
    ivFiles->setMaxItemWidth( 32 );
    ivFiles->setItemsMovable( false );
    ivFiles->setWordWrapIconText( true );
    ivFiles->setSorting( true, true );
    ivFiles->setSelectionMode( QIconView::Extended );

    filesIconsMenu = new QPopupMenu( ivFiles );

    ivFilesThumbnails = new KatIconView( this );
    ivFilesThumbnails->hide();
    ivFilesThumbnails->setResizeMode( QIconView::Adjust );
    ivFilesThumbnails->setItemTextPos( QIconView::Bottom );
    ivFilesThumbnails->setGridX( 80 );
    ivFilesThumbnails->setGridY( 80 );
    ivFilesThumbnails->setMaxItemWidth( 160 );
    ivFilesThumbnails->setItemsMovable( false );
    ivFilesThumbnails->setWordWrapIconText( true );
    ivFilesThumbnails->setSorting( true, true );
    ivFilesThumbnails->setSelectionMode( QIconView::Extended );

    filesThumbnailsMenu = new QPopupMenu( ivFilesThumbnails );

    dockFiles->setWidget( lvFiles );
    dockMain = dockFiles->manualDock( dockFolders, KDockWidget::DockRight, 2000 );

    // Properties dock
    dockProperties = createDockWidget( "properties_view", NULL, 0L,
                                       kapp->makeStdCaption( "Properties view" ),
                                       "Properties view" );
    teProperties = new KTextEdit( this );
    teProperties->setReadOnly( true );
    teProperties->setPaletteBackgroundPixmap( QPixmap( appDataPath ) );

    dockProperties->setWidget( teProperties );
    dockProperties->manualDock( dockFiles, KDockWidget::DockBottom, 6000 );

    // MainDockWidget
    setView( dockMain );
    setMainDockWidget( dockMain );

    // Search dock
    dockSearch = createDockWidget( "search_view", NULL, 0L,
                                   kapp->makeStdCaption( "Search view" ),
                                   "Search view" );
    dsSearch = new DlgSearch( this );
    dockSearch->setWidget( dsSearch );

    // Results dock
    dockResults = createDockWidget( "results_view", NULL, 0L,
                                    kapp->makeStdCaption( "Results view" ),
                                    "Results view" );
    lvResults = new KatListView( this );
    lvResults->hide();
    lvResults->addColumn( i18n( "Filename" ) );
    lvResults->addColumn( i18n( "Ext" ) );
    lvResults->addColumn( i18n( "Type" ) );
    lvResults->addColumn( i18n( "Size" ) );
    lvResults->addColumn( i18n( "Date" ) );
    lvResults->addColumn( i18n( "Time" ) );
    lvResults->addColumn( i18n( "Thumbnail" ) );
    lvResults->addColumn( i18n( "Metadata" ) );
    lvResults->addColumn( i18n( "Fulltext" ) );
    lvResults->addColumn( i18n( "Language" ) );
    lvResults->setSorting( FILE_NAME, true );
    lvResults->setAllColumnsShowFocus( true );
    lvResults->setRootIsDecorated( false );
    lvResults->setShowSortIndicator( true );
    lvResults->setColumnAlignment( FILE_NAME, Qt::AlignLeft );
    lvResults->setColumnAlignment( FILE_EXTENSION, Qt::AlignLeft );
    lvResults->setColumnAlignment( FILE_MIMETYPE, Qt::AlignLeft );
    lvResults->setColumnAlignment( FILE_SIZE, Qt::AlignRight );
    lvResults->setColumnAlignment( FILE_DATE, Qt::AlignRight );
    lvResults->setColumnAlignment( FILE_TIME, Qt::AlignRight );
    lvResults->setColumnAlignment( FILE_THUMBNAIL, Qt::AlignCenter );
    lvResults->setColumnAlignment( FILE_METADATA, Qt::AlignCenter );
    lvResults->setColumnAlignment( FILE_FULLTEXT, Qt::AlignCenter );
    lvResults->setColumnAlignment( FILE_LANGUAGE, Qt::AlignLeft );
    lvResults->header()->setLabel( FILE_NAME, i18n( "Filename" ) );
    lvResults->header()->setLabel( FILE_EXTENSION, i18n( "Ext" ) );
    lvResults->header()->setLabel( FILE_MIMETYPE, i18n( "Type" ) );
    lvResults->header()->setLabel( FILE_SIZE, i18n( "Size" ) );
    lvResults->header()->setLabel( FILE_DATE, i18n( "Date" ) );
    lvResults->header()->setLabel( FILE_TIME, i18n( "Time" ) );
    lvResults->header()->setLabel( FILE_THUMBNAIL, i18n( "Thumbnail" ) );
    lvResults->header()->setLabel( FILE_METADATA, i18n( "Metadata" ) );
    lvResults->header()->setLabel( FILE_FULLTEXT, i18n( "Fulltext" ) );
    lvResults->header()->setLabel( FILE_LANGUAGE, i18n( "Language" ) );
    lvResults->setSorting( FILE_NAME, true );
    lvResults->setSelectionMode( QListView::Extended );

    resultsMenu = new QPopupMenu( lvResults );

    ivResults = new KatIconView( this );
    ivResults->hide();
    ivResults->setResizeMode( QIconView::Adjust );
    ivResults->setItemTextPos( QIconView::Bottom );
    ivResults->setGridX( 80 );
    ivResults->setGridY( 80 );
    ivResults->setMaxItemWidth( 32 );
    ivResults->setItemsMovable( false );
    ivResults->setWordWrapIconText( true );
    ivResults->setSorting( true, true );
    ivResults->setSelectionMode( QIconView::Extended );

    resultsIconsMenu = new QPopupMenu( ivResults );

    ivResultsThumbnails = new KatIconView( this );
    ivResultsThumbnails->hide();
    ivResultsThumbnails->setResizeMode( QIconView::Adjust );
    ivResultsThumbnails->setItemTextPos( QIconView::Bottom );
    ivResultsThumbnails->setGridX( 80 );
    ivResultsThumbnails->setGridY( 80 );
    ivResultsThumbnails->setMaxItemWidth( 160 );
    ivResultsThumbnails->setItemsMovable( false );
    ivResultsThumbnails->setWordWrapIconText( true );
    ivResultsThumbnails->setSorting( true, true );
    ivResultsThumbnails->setSelectionMode( QIconView::Extended );

    resultsThumbnailsMenu = new QPopupMenu( ivResultsThumbnails );

    dockResults->setWidget( lvResults );
    dockMainSearch = dockResults->manualDock( dockSearch, KDockWidget::DockBottom, 5000 );

    // FINISHING
    resize( QSize( 800, 600 ).expandedTo( minimumSizeHint() ) );
    clearWState( WState_Polished );
}

void DlgMainWindow::slotFileNew() {
    fileNew();
}

bool DlgMainWindow::fileNew() {

    if ( m_kat && m_kat->getFilename().isEmpty() ) {
        int result = KMessageBox::questionYesNoCancel( this, "The current catalog has not been saved yet.\n "
                                                                "Do you want to save it before creating a new catalog?",
                                                                "Warning" );

        switch (result) {
            case KMessageBox::Cancel:
                break;
            case KMessageBox::Yes:
                fileSave();
                createNewCatalog();
                break;
            case KMessageBox::No:
                createNewCatalog();
                break;
        }

    } else {
        createNewCatalog();
    }

    return false;
}

QString DlgMainWindow::katFilename()
{
    if ( m_kat && !m_kat->getFilename().isEmpty() )
        return m_kat->getFilename();
    else
        return QString::null;
}


void DlgMainWindow::saveProperties( KConfig *config )
{
    kdDebug()<<"void DlgMainWindow::saveProperties( KConfig *config )" << endl;
    KURL url;
    if ( m_kat && !m_kat->getFilename().isEmpty() )
        url = KURL( m_kat->getFilename() );
    config->writePathEntry( "URL" , url.url() );
}

void DlgMainWindow::readProperties( KConfig *config )
{
    kdDebug()<<"void DlgMainWindow::readProperties( KConfig *config )" << endl;
    fileOpen( KURL( config->readPathEntry( "URL" ) ) );
}

void DlgMainWindow::createNewCatalog() {
    DlgNewCatalog nc( this );

    if ( nc.exec() == QDialog::Accepted )
    {

        DlgCreateCatalog cc( this );

        if ( cc.exec() == QDialog::Accepted )
        {
            populateViews();
            toolbarSearch->setEnabled( true );
            toolbarConfigure->setEnabled( true );
            toolbarViewDetailed->setEnabled( true );
            toolbarViewIcons->setEnabled( true );
            toolbarViewThumbnails->setEnabled( true );
            fileSaveAction->setEnabled( true );
            fileSaveAsAction->setEnabled( true );

        }
    }
}

void DlgMainWindow::slotFileOpen() {
    fileOpen();
}

void DlgMainWindow::fileOpen( const KURL &url )
{
    //TODO test if we have already a file opened
    struct stat my_stat;
    if ( !url.isEmpty() ) {

        QString filename = url.path();

        // Check if the file exists
        int result = stat( filename, &my_stat );
        if ( result == -1 ) {
            KMessageBox::error( this, i18n( "The specified file does not exist or is not accessible" ) );
            return;
        }

        m_kat = new KatCatalog();

        dm->openKatDatabase( filename );
        dm->readCatalogInformation( *m_kat );
        m_kat->setFilename( filename );

        populateViews();
        toolbarSearch->setEnabled( true );
        toolbarConfigure->setEnabled( true );
        toolbarViewDetailed->setEnabled( true );
        toolbarViewIcons->setEnabled( true );
        toolbarViewThumbnails->setEnabled( true );
        fileSaveAction->setEnabled( false );
        fileSaveAsAction->setEnabled( true );
    }
}

bool DlgMainWindow::fileOpen() {

    if ( m_kat && m_kat->getFilename().isEmpty() ) {
        int result = KMessageBox::questionYesNoCancel( this, i18n( "The current catalog has not been saved yet.\n "
                                                                   "Do you want to save it before opening another catalog?",
                                                                   "Warning" ) );

        switch (result) {
            case KMessageBox::Cancel:
                break;
            case KMessageBox::Yes:
                fileSave();
                openCatalog();
                break;
            case KMessageBox::No:
                openCatalog();
                break;
        }

    } else {
        openCatalog();
    }

    return false;
}

void DlgMainWindow::openCatalog() {
    KURL fn = KFileDialog::getOpenURL( QString::null, "*.kat|" + i18n( "Kat catalogs (*.kat)" ),
                                       this, i18n( "Select the catalog to open" ) );

    if ( !fn.isEmpty() ) {
        fileOpen( fn );
        fileOpenRecent->addURL( fn );
    }
}

void DlgMainWindow::fileSave() {
    if ( m_kat && m_kat->getFilename().isEmpty() )
    {
        QString fn = KFileDialog::getSaveFileName( QString::null, "*.kat|"+ i18n( "Kat catalogs (*.kat)" ), this ,
                                                   i18n( "Specify location and filename of the catalog" ));
        if ( !fn.isEmpty() )
        {
            m_kat->save( QString::null, fn );
            fileSaveAction->setEnabled( false );
        }
    }
}

void DlgMainWindow::slotFileSave() {
    fileSave();
}

void DlgMainWindow::slotFileSaveAs() {
    fileSaveAs();
}

bool DlgMainWindow::fileSaveAs() {
    if ( m_kat  ) {
        QString katFilename = m_kat->getFilename();

        QString fn = KFileDialog::getSaveFileName( QString::null, "*.kat|" +i18n( "Kat catalogs (*.kat)" ), this ,
                                                   i18n( "Specify location and filename of the catalog" ));
        if ( !fn.isEmpty() ) {
            m_kat->save( katFilename, fn );
            return false;
        } else {
            return true;
        }
    }

    return false;
}

void DlgMainWindow::slotFilePrint() {
}

void DlgMainWindow::slotSaveProperties() {
    fileOpenRecent->saveEntries( m_config );
}


bool DlgMainWindow::queryClose() {
    if ( m_kat && m_kat->getFilename().isEmpty() ) {

        int result = KMessageBox::questionYesNo( this, "The current catalog has not been saved yet.\n "
                                                                "Do you want to save ?",
                                                                "Warning" );
        if ( result==KMessageBox::Yes )
        {

            QString fn = KFileDialog::getSaveFileName( QString::null, "*.kat|"+i18n( "Kat catalogs (*.kat)" ), this ,
                                                       i18n("Specify location and filename of the catalog" ));
            if ( !fn.isEmpty() ) {
                m_kat->save( QString::null, fn );
                return true;
            }
        }
        else
            return true;
    }
    return true;
}

void DlgMainWindow::slotEditUndo() {
}

void DlgMainWindow::slotEditRedo() {
}

void DlgMainWindow::slotEditCut() {
}

void DlgMainWindow::slotEditCopy() {
}

void DlgMainWindow::slotEditPaste() {
}

void DlgMainWindow::slotEditFind() {
}

void DlgMainWindow::slotConfigureKeys() {
  KKeyDialog::configure( actionCollection(), this );
}

void DlgMainWindow::slotViewDetailed() {
    dockFiles->setWidget( lvFiles );
    lvFiles->show();
    ivFiles->hide();
    ivFilesThumbnails->hide();

    dockResults->setWidget( lvResults );
    lvResults->show();
    ivResults->hide();
    ivResultsThumbnails->hide();
}

void DlgMainWindow::slotViewIcons() {
    dockFiles->setWidget( ivFiles );
    ivFiles->show();
    lvFiles->hide();
    ivFilesThumbnails->hide();

    dockResults->setWidget( ivResults );
    ivResults->show();
    lvResults->hide();
    ivResultsThumbnails->hide();
}

void DlgMainWindow::slotViewThumbnails() {
    dockFiles->setWidget( ivFilesThumbnails );
    ivFilesThumbnails->show();
    lvFiles->hide();
    ivFiles->hide();

    dockResults->setWidget( ivResultsThumbnails );
    ivResultsThumbnails->show();
    lvResults->hide();
    ivResults->hide();
}

void DlgMainWindow::expandFolder( QListViewItem * item ) {
    if ( item ) {
        KatListViewItem* lvItem = static_cast<KatListViewItem*>(item);

        DlgMainWindow::insertChildrenFolders( lvItem->info.fileId, item );
    }
}

void DlgMainWindow::selectFolder( QListViewItem* item ) {
    if ( item ) {
        KatListViewItem* lvItem = static_cast<KatListViewItem*>(item);

        QString fileType = lvItem->info.fileType;
        long fileId = lvItem->info.fileId;

        int columns = lvFiles->columns();

        if ( fileType == "KAT_CATALOG" && prevFileType != "KAT_CATALOG" ) {

            for ( int i=0; i <= columns; i++ ) {
                lvFiles->removeColumn( 0 );
            }

            lvFiles->addColumn( i18n( "Catalog name" ) );
            lvFiles->addColumn( i18n( "Base path" ) );
            lvFiles->addColumn( i18n( "Author" ) );
            lvFiles->addColumn( i18n( "Creation date" ) );
            lvFiles->addColumn( i18n( "Last update" ) );
            lvFiles->addColumn( i18n( "Files" ) );
            lvFiles->addColumn( i18n( "Words" ) );
            lvFiles->setSorting( CATALOG_NAME, true );
            lvFiles->setAllColumnsShowFocus( true );
            lvFiles->setRootIsDecorated( true );
            lvFiles->setShowSortIndicator( true );
            lvFiles->setColumnAlignment( CATALOG_NAME, Qt::AlignLeft );
            lvFiles->setColumnAlignment( CATALOG_BASEPATH, Qt::AlignLeft );
            lvFiles->setColumnAlignment( CATALOG_AUTHOR, Qt::AlignLeft );
            lvFiles->setColumnAlignment( CATALOG_CREATIONDATE, Qt::AlignRight );
            lvFiles->setColumnAlignment( CATALOG_LASTUPDATE, Qt::AlignRight );
            lvFiles->setColumnAlignment( CATALOG_FILES, Qt::AlignRight );
            lvFiles->setColumnAlignment( CATALOG_WORDS, Qt::AlignRight );

            QDateTime cdt;
            cdt.setTime_t( m_kat->getCreationDate() );
            QDate cdate = cdt.date();
            QTime ctime = cdt.time();

            QString creationdate = cdate.toString( Qt::LocalDate ) + " " + ctime.toString( Qt::TextDate );

            QDateTime ludt;
            ludt.setTime_t( m_kat->getLastUpdateDate() );
            QDate ludate = ludt.date();
            QTime lutime = ludt.time();

            QString lastupdatedate = ludate.toString( Qt::LocalDate ) + " " + lutime.toString( Qt::TextDate );

            lvFiles->clear();
            ivFiles->clear();
            ivFilesThumbnails->clear();
            teProperties->setText( QString::null );

            QString files = QString::number( m_kat->getFiles() );
            addSeparators( files );

            QString words = QString::number( m_kat->getWords() );
            addSeparators( words );

            KatListViewItem* lvItem = new KatListViewItem( lvFiles );
            lvItem->setText( CATALOG_NAME, m_kat->getName() );
            lvItem->setText( CATALOG_BASEPATH, m_kat->getPath() );
            lvItem->setText( CATALOG_AUTHOR, m_kat->getAuthor() );
            lvItem->setText( CATALOG_CREATIONDATE, creationdate );
            lvItem->setText( CATALOG_LASTUPDATE, lastupdatedate );
            lvItem->setText( CATALOG_FILES, files );
            lvItem->setText( CATALOG_WORDS, words );

            KatInformation info;
            info.fileType = "KAT_CATALOG";
            info.fileId = -2L;
            lvItem->info = info;

        } else if ( fileType != "KAT_CATALOG"  && prevFileType == "KAT_CATALOG" ) {
            for ( int i=0; i <= columns; i++ ) {
                lvFiles->removeColumn( 0 );
            }

            lvFiles->addColumn( i18n( "Filename" ) );
            lvFiles->addColumn( i18n( "Ext" ) );
            lvFiles->addColumn( i18n( "Type" ) );
            lvFiles->addColumn( i18n( "Size" ) );
            lvFiles->addColumn( i18n( "Date" ) );
            lvFiles->addColumn( i18n( "Time" ) );
            lvFiles->addColumn( i18n( "Thumbnail" ) );
            lvFiles->addColumn( i18n( "Metadata" ) );
            lvFiles->addColumn( i18n( "Fulltext" ) );
            lvFiles->addColumn( i18n( "Language" ) );
            lvFiles->setSorting( FILE_NAME, true );
            lvFiles->setAllColumnsShowFocus( true );
            lvFiles->setRootIsDecorated( true );
            lvFiles->setShowSortIndicator( true );
            lvFiles->setColumnAlignment( FILE_NAME, Qt::AlignLeft );
            lvFiles->setColumnAlignment( FILE_EXTENSION, Qt::AlignLeft );
            lvFiles->setColumnAlignment( FILE_MIMETYPE, Qt::AlignLeft );
            lvFiles->setColumnAlignment( FILE_SIZE, Qt::AlignRight );
            lvFiles->setColumnAlignment( FILE_DATE, Qt::AlignRight );
            lvFiles->setColumnAlignment( FILE_TIME, Qt::AlignRight );
            lvFiles->setColumnAlignment( FILE_THUMBNAIL, Qt::AlignCenter );
            lvFiles->setColumnAlignment( FILE_METADATA, Qt::AlignCenter );
            lvFiles->setColumnAlignment( FILE_FULLTEXT, Qt::AlignCenter );
            lvFiles->setColumnAlignment( FILE_LANGUAGE, Qt::AlignLeft );

            lvFiles->clear();
            ivFiles->clear();
            ivFilesThumbnails->clear();
            teProperties->setText( QString::null );

            insertChildrenFiles( fileId );
        } else {
            lvFiles->clear();
            ivFiles->clear();
            ivFilesThumbnails->clear();
            teProperties->setText( QString::null );

            insertChildrenFiles( fileId );
        }

        prevFileType = fileType;
    }
}

void DlgMainWindow::selectFile( QListViewItem* item ) {
    if ( item ) {
        KatListViewItem* lvItem = static_cast<KatListViewItem*>(item);
        showFileProperties( lvItem->info );
    }
}

void DlgMainWindow::selectFileIcon(QIconViewItem* item) {
    if ( item ) {
        KatIconViewItem* lvItem = static_cast<KatIconViewItem*>(item);
        showFileProperties( lvItem->info );
    }
}

void DlgMainWindow::selectFileThumbnail( QIconViewItem* item ) {
    if ( item ) {
        KatIconViewItem* lvItem = static_cast<KatIconViewItem*>(item);
        showFileProperties( lvItem->info );
    }
}

void DlgMainWindow::contextFile( KListView* lv, QListViewItem* item, const QPoint& point ) {

    QPtrList<QListViewItem> lvItems = lv->selectedItems();

    // check if all lvItems have the same mimeType
    QString mimeType = QString::null;
    bool different = false;
    for ( QPtrList<QListViewItem>::iterator it = lvItems.begin(); it != lvItems.end(); it++ ) {
        KatListViewItem* lvItem = static_cast<KatListViewItem*>(*it);
        QString fileMimeType = lvItem->info.fileType;
        if ( ( mimeType != "" ) && ( mimeType != fileMimeType ) ) {
            different = true;
            break;
        }
        mimeType = fileMimeType;
    }

    filesMenu->clear();

    QPopupMenu* openWithMenu = new QPopupMenu();

    bool addOpenWithMenu = true;
    if ( different ) {
        openWithMenu->setEnabled( false );
    } else {
        KMimeType::Ptr mime = KMimeType::mimeType( mimeType );

        offers = KTrader::self()->query( mime->name(), "Type == 'Application'" );

        if ( offers.count()>0 )
        {
            connect( openWithMenu, SIGNAL( activated( int ) ), this, SLOT( slotOpenWith( int ) )  );


            KTrader::OfferList::Iterator iter;
            KService::Ptr ptr;
            int index = 100;
            for( iter = offers.begin(); iter != offers.end(); ++iter ) {
                ptr = *iter;
                openWithMenu->insertItem( ptr->pixmap( KIcon::Small ), ptr->name(), index++ );
            }
        }
        else
            addOpenWithMenu=false;
    }
    if ( addOpenWithMenu )
        filesMenu->insertItem( i18n( "Open With" ), openWithMenu, 1 );

    // The following menu items are enabled if 1 only file is selected
    if ( lvItems.count() == 1 ) {
        KatListViewItem* lvItem = static_cast<KatListViewItem*>(lvItems.first());
        if ( lvItem->info.hasFullText ) {
            if ( addOpenWithMenu )
                filesMenu->insertSeparator();
            filesMenu->insertItem( i18n( "Show fulltext" ), this, SLOT( slotShowFullText() ), 0, 2 );
            filesMenu->insertItem( i18n( "Show words list" ), this, SLOT( slotShowWordsList() ), 0, 3 );
            filesMenu->setItemEnabled ( 2, true );
            filesMenu->setItemEnabled ( 3, true );
        }
    } else {
        if ( addOpenWithMenu )
            filesMenu->insertSeparator();

        filesMenu->insertItem( i18n( "Show fulltext" ), 2 );
        filesMenu->insertItem( i18n( "Show words list" ), 3 );
        filesMenu->setItemEnabled ( 2, false );
        filesMenu->setItemEnabled ( 3, false );
    }
    filesMenu->popup( point );
}

void DlgMainWindow::slotOpenWith( int id ) {
    if ( id >= 100 && id < 1000 ) {

        QPtrList<QListViewItem> lvItems;
        QPtrList<QIconViewItem> ivItems;

        if ( getMainDockWidget() == dockMain ) {

            if ( !lvFiles->isHidden() ) {
                lvItems = lvFiles->selectedItems();
            }
            if ( !ivFiles->isHidden() ) {
                for ( QIconViewItem *item = ivFiles->firstItem(); item; item = item->nextItem() ) {
                    if ( item->isSelected() ) {
                        ivItems.append( item );
                    }
                }
            }
            if ( !ivFilesThumbnails->isHidden() ) {
                for ( QIconViewItem *item = ivFilesThumbnails->firstItem(); item; item = item->nextItem() ) {
                    if ( item->isSelected() ) {
                        ivItems.append( item );
                    }
                }
            }
        } else {
            if ( !lvResults->isHidden() ) {
                lvItems = lvResults->selectedItems();
            }
            if ( !ivResults->isHidden() ) {
                for ( QIconViewItem *item = ivResults->firstItem(); item; item = item->nextItem() ) {
                    if ( item->isSelected() ) {
                        ivItems.append( item );
                    }
                }
            }
            if ( !ivResultsThumbnails->isHidden() ) {
                for ( QIconViewItem *item = ivResultsThumbnails->firstItem(); item; item = item->nextItem() ) {
                    if ( item->isSelected() ) {
                        ivItems.append( item );
                    }
                }
            }
        }

        KTrader::OfferList::Iterator iter;
        KService::Ptr ptr;
        int index = 100;
        for( iter = offers.begin(); iter != offers.end(); ++iter ) {
            ptr = *iter;
            if ( index == id ) {
                //kdDebug() << ptr->name() << endl;

                KURL::List urlList;

                if ( lvItems.count() != 0 ) {
                    for ( QPtrList<QListViewItem>::iterator it = lvItems.begin(); it != lvItems.end(); it++ ) {
                        KatListViewItem* lvItem = static_cast<KatListViewItem*>(*it);
                        urlList.append( KURL( lvItem->info.fullName ) );
                    }
                }
                if ( ivItems.count() != 0 ) {
                    for ( QPtrList<QIconViewItem>::iterator it = ivItems.begin(); it != ivItems.end(); it++ ) {
                        KatIconViewItem* ivItem = static_cast<KatIconViewItem*>(*it);
                        urlList.append( KURL( ivItem->info.fullName ) );
                    }
                }

                if ( urlList.count() ) {
                    KRun::run( *ptr, urlList );
                }

                break;
            }
            index++;
        }
        offers.clear();
    }
}

void DlgMainWindow::contextFileIcon( QIconViewItem* item, const QPoint& point ) {
    QPtrList<QIconViewItem> ivItems;

    if ( getMainDockWidget() == dockMain ) {
        for ( QIconViewItem *item = ivFiles->firstItem(); item; item = item->nextItem() ) {
            if ( item->isSelected() ) {
                ivItems.append( item );
            }
        }
    } else {
        for ( QIconViewItem *item = ivResults->firstItem(); item; item = item->nextItem() ) {
            if ( item->isSelected() ) {
                ivItems.append( item );
            }
        }
    }

    // check if all lvItems have the same mimeType
    QString mimeType = QString::null;
    bool different = false;
    for ( QPtrList<QIconViewItem>::iterator it = ivItems.begin(); it != ivItems.end(); it++ ) {
        KatIconViewItem* ivItem = static_cast<KatIconViewItem*>(*it);
        QString fileMimeType = ivItem->info.fileType;
        if ( ( mimeType != "" ) && ( mimeType != fileMimeType ) ) {
            different = true;
            break;
        }
        mimeType = fileMimeType;
    }

    filesIconsMenu->clear();

    QPopupMenu* openWithMenu = new QPopupMenu();

    bool addOpenWithMenu = true;
    if ( different ) {
        openWithMenu->setEnabled( false );
    } else {
        KMimeType::Ptr mime = KMimeType::mimeType( mimeType );

        offers = KTrader::self()->query( mime->name(), "Type == 'Application'" );

        if ( offers.count() > 0 )
        {
            connect( openWithMenu, SIGNAL( activated( int ) ), this, SLOT( slotOpenWith( int ) )  );

            KTrader::OfferList::Iterator iter;
            KService::Ptr ptr;
            int index = 100;
            for( iter = offers.begin(); iter != offers.end(); ++iter ) {
                ptr = *iter;
                openWithMenu->insertItem( ptr->pixmap( KIcon::Small ), ptr->name(), index++ );
            }
        }
        else
            addOpenWithMenu = false;
    }
    if ( addOpenWithMenu )
        filesIconsMenu->insertItem( i18n( "Open With" ), openWithMenu, 1 );

    // The following menu items are enabled if 1 only file is selected
    if ( ivItems.count() == 1 ) {
        KatIconViewItem* ivItem = static_cast<KatIconViewItem*>(ivItems.first());
        if ( ivItem->info.hasFullText ) {
            if ( addOpenWithMenu )
                filesIconsMenu->insertSeparator();
            filesIconsMenu->insertItem( i18n( "Show fulltext" ), this, SLOT( slotShowFullText() ), 0, 2 );
            filesIconsMenu->insertItem( i18n( "Show words list" ), this, SLOT( slotShowWordsList() ), 0, 3 );
            filesIconsMenu->setItemEnabled ( 2, true );
            filesIconsMenu->setItemEnabled ( 3, true );
        }
    } else {
        if ( addOpenWithMenu )
            filesIconsMenu->insertSeparator();

        filesIconsMenu->insertItem( i18n( "Show fulltext" ), 2 );
        filesIconsMenu->insertItem( i18n( "Show words list" ), 3 );
        filesIconsMenu->setItemEnabled ( 2, false );
        filesIconsMenu->setItemEnabled ( 3, false );
    }
    filesIconsMenu->popup( point );
}

void DlgMainWindow::contextFileThumbnail( QIconViewItem* item, const QPoint& point ) {
     QPtrList<QIconViewItem> ivItems;

    if ( getMainDockWidget() == dockMain ) {
        for ( QIconViewItem *item = ivFilesThumbnails->firstItem(); item; item = item->nextItem() ) {
            if ( item->isSelected() ) {
                ivItems.append( item );
            }
        }
    } else {
        for ( QIconViewItem *item = ivResultsThumbnails->firstItem(); item; item = item->nextItem() ) {
            if ( item->isSelected() ) {
                ivItems.append( item );
            }
        }
    }

    // check if all lvItems have the same mimeType
    QString mimeType = QString::null;
    bool different = false;
    for ( QPtrList<QIconViewItem>::iterator it = ivItems.begin(); it != ivItems.end(); it++ ) {
        KatIconViewItem* ivItem = static_cast<KatIconViewItem*>(*it);
        QString fileMimeType = ivItem->info.fileType;
        if ( ( mimeType != "" ) && ( mimeType != fileMimeType ) ) {
            different = true;
            break;
        }
        mimeType = fileMimeType;
    }

    filesThumbnailsMenu->clear();

    QPopupMenu* openWithMenu = new QPopupMenu();

    bool addOpenWithMenu = true;
    if ( different ) {
        openWithMenu->setEnabled( false );
    } else {
        KMimeType::Ptr mime = KMimeType::mimeType( mimeType );

        offers = KTrader::self()->query( mime->name(), "Type == 'Application'" );

        if ( offers.count() > 0 )
        {
            connect( openWithMenu, SIGNAL( activated( int ) ), this, SLOT( slotOpenWith( int ) )  );

            KTrader::OfferList::Iterator iter;
            KService::Ptr ptr;
            int index = 100;
            for( iter = offers.begin(); iter != offers.end(); ++iter ) {
                ptr = *iter;
                openWithMenu->insertItem( ptr->pixmap( KIcon::Small ), ptr->name(), index++ );
            }
        }
        else
            addOpenWithMenu = false;
    }
    if ( addOpenWithMenu )
        filesThumbnailsMenu->insertItem( i18n( "Open With" ), openWithMenu, 1 );

    // The following menu items are enabled if 1 only file is selected
    if ( ivItems.count() == 1 ) {
        KatIconViewItem* ivItem = static_cast<KatIconViewItem*>(ivItems.first());
        if ( ivItem->info.hasFullText ) {
            if ( addOpenWithMenu )
                filesThumbnailsMenu->insertSeparator();
            filesThumbnailsMenu->insertItem( i18n( "Show fulltext" ), this, SLOT( slotShowFullText() ), 0, 2 );
            filesThumbnailsMenu->insertItem( i18n( "Show words list" ), this, SLOT( slotShowWordsList() ), 0, 3 );
            filesThumbnailsMenu->setItemEnabled ( 2, true );
            filesThumbnailsMenu->setItemEnabled ( 3, true );
        }
    } else {
        if ( addOpenWithMenu )
            filesThumbnailsMenu->insertSeparator();

        filesThumbnailsMenu->insertItem( i18n( "Show fulltext" ), 2 );
        filesThumbnailsMenu->insertItem( i18n( "Show words list" ), 3 );
        filesThumbnailsMenu->setItemEnabled ( 2, false );
        filesThumbnailsMenu->setItemEnabled ( 3, false );
    }
    filesThumbnailsMenu->popup( point );
}

void DlgMainWindow::contextReqFile( QListViewItem* item, const QPoint& point, int i ) {
    kdDebug() << i << endl;
}

void DlgMainWindow::slotShowFullText() {
    KatListViewItem* lvItem = 0L;
    KatIconViewItem* ivItem = 0L;

    if ( getMainDockWidget() == dockMain ) {
        if ( !lvFiles->isHidden() ) {
            lvItem = static_cast<KatListViewItem*>(lvFiles->selectedItems().first());
        }
        if ( !ivFiles->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivFiles->currentItem());
        }
        if ( !ivFilesThumbnails->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivFilesThumbnails->currentItem());
        }
    } else {
        if ( !lvResults->isHidden() ) {
            lvItem = static_cast<KatListViewItem*>(lvResults->selectedItems().first());
        }
        if ( !ivResults->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivResults->currentItem());
        }
        if ( !ivResultsThumbnails->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivResultsThumbnails->currentItem());
        }
    }

    if ( lvItem ) {
        DlgShowFullText sft( this, lvItem->info.fileId );
        sft.exec();
    }
    if ( ivItem ) {
        DlgShowFullText sft( this, ivItem->info.fileId );
        sft.exec();
    }
}

void DlgMainWindow::slotShowWordsList() {
    KatListViewItem* lvItem = 0L;
    KatIconViewItem* ivItem = 0L;

    if ( getMainDockWidget() == dockMain ) {
        if ( !lvFiles->isHidden() ) {
            lvItem = static_cast<KatListViewItem*>(lvFiles->selectedItems().first());
        }
        if ( !ivFiles->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivFiles->currentItem());
        }
        if ( !ivFilesThumbnails->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivFilesThumbnails->currentItem());
        }
    } else {
        if ( !lvResults->isHidden() ) {
            lvItem = static_cast<KatListViewItem*>(lvResults->selectedItems().first());
        }
        if ( !ivResults->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivResults->currentItem());
        }
        if ( !ivResultsThumbnails->isHidden() ) {
            ivItem = static_cast<KatIconViewItem*>(ivResultsThumbnails->currentItem());
        }
    }

    if ( lvItem ) {
        DlgShowWordsList swl( this, lvItem->info.fileId );
        swl.exec();
    }
    if ( ivItem ) {
        DlgShowWordsList swl( this, ivItem->info.fileId );
        swl.exec();
    }
}

void DlgMainWindow::contextFolder( KListView*, QListViewItem*, const QPoint& ) {
}

void DlgMainWindow::contextReqFolder( QListViewItem* item, const QPoint& point, int i ) {
}

void DlgMainWindow::openItemSelected( KatListViewItem* item ) {
    if ( item ) {
        if ( item->info.fileType != "KAT_CATALOG" ) {
            openFile( item->info );
        }
    }
}

void DlgMainWindow::openItemSelected( KatIconViewItem* item ) {
    if ( item ) {
        if ( item->info.fileType != "KAT_CATALOG" ) {
            openFile( item->info );
        }
    }
}

void DlgMainWindow::doubleClickedFile( QListViewItem* item ) {
    KatListViewItem* lvItem = static_cast<KatListViewItem*>(item);
    openItemSelected( lvItem );
}

void DlgMainWindow::doubleClickedFileIcon( QIconViewItem* item ) {
    KatIconViewItem* lvItem = static_cast<KatIconViewItem*>(item);
    openItemSelected( lvItem );
}

void DlgMainWindow::doubleClickedFileThumbnail( QIconViewItem* item ) {
    KatIconViewItem* lvItem = static_cast<KatIconViewItem*>(item);
    openItemSelected( lvItem );
}

void DlgMainWindow::doubleClickedFolder( QListViewItem* item ) {
    if ( item ) {
        if ( item->isOpen() ) {
            item->setOpen( false );
        } else {
            item->setOpen( true );
        }
    }
}

void DlgMainWindow::rightButtonFile( QListViewItem* item, const QPoint&, int ) {
}

void DlgMainWindow::rightButtonFolder( QListViewItem*, const QPoint&, int ) {
}

void DlgMainWindow::executeFile( QListViewItem* ) {
}

void DlgMainWindow::executeFolder( QListViewItem* ) {
}

void DlgMainWindow::slotFileFind() {
    if ( getMainDockWidget() == dockMain ) {
        makeDockInvisible( dockMain );

        setView( dockMainSearch );
        setMainDockWidget( dockMainSearch );

        makeDockVisible( dockMainSearch );
    } else {
        makeDockInvisible( dockMainSearch );

        setView( dockMain );
        setMainDockWidget( dockMain );

        makeDockVisible( dockMain );
    }
}

void DlgMainWindow::insertChildrenFolders( long parentId, QListViewItem* parentItem ) {
    QMap<long,QString> children;
    QString fileName;
    QString fileType;
    long fileId;

    // eliminate eventual children
    while( parentItem->firstChild() ) {
        delete parentItem->firstChild();
    }

    // rebuild the list of children
    children = dm->getFileChildren( parentId );

    QMap<long,QString>::iterator i;
    for ( i = children.begin(); i != children.end(); ++i ) {

        fileId = i.key();
        fileName = i.data();

        unsigned pos = fileName.findRev( '/', -1 );
        fileName = fileName.right( fileName.length() - pos - 1 );

        KatInformation info = dm->readFileInformation( fileId );

        if ( info.fileType == "inode/directory" ) {
            KatListViewItem* lvItem = new KatListViewItem( parentItem );
            lvItem->setText( 0, fileName );
            lvItem->setPixmap( 0, pmFolderIconSmall );
            lvItem->setOpen( false );
            lvItem->info = info;

            // If the item has children, insert a fake child in order to show the + sign
            if ( dm->folderHasChildren( fileId ) ) {
                KatListViewItem* lvFakeItem = new KatListViewItem( lvItem );
            }
        }
    }
}

void DlgMainWindow::insertChildrenFiles( long parentId ) {
    QMap<long,QString> children;
    QString fileName;
    QString fullName;
    QString ext;
    long fileId;
    QString language;

    children = dm->getFileChildren( parentId );

    QMap<long,QString>::iterator i;
    for ( i = children.begin(); i != children.end(); ++i ) {

        fileId = i.key();
        fullName = i.data();

        KatInformation info = dm->readFileInformation( fileId );

        language = info.language;

        int pos = fullName.findRev( '/', -1 );
        fileName = fullName.right( fullName.length() - pos - 1 );

        KMimeType::Ptr mime = KMimeType::mimeType( info.fileType );
        QString mimeType = info.fileType;

        pos = fileName.findRev( '.', -1 );
        if ( pos == -1 ) {
            ext = QString::null;
        } else {
            ext = fileName.right( fileName.length() - pos - 1 );
            ext = ext.lower();
        }

        children.insert( i.key(), fileName  );

        QDateTime dt;
        dt.setTime_t( info.modificationDate );
        QDate date = dt.date();
        QTime time = dt.time();

        KatListViewItem *lvItem = new KatListViewItem( lvFiles );
        lvItem->setDragEnabled( true );
        lvItem->info = info;
        KatIconViewItem *ivItem = new KatIconViewItem( ivFiles );
        ivItem->info = info;
        KatIconViewItem *ivThmb = new KatIconViewItem( ivFilesThumbnails );
        ivThmb->info = info;

        if ( info.fileType == "inode/directory" ) {

            const QPixmap& pmFolderIconSmall =
                KMimeType::mimeType( "inode/directory" )->pixmap( KIcon::Small );
            const QPixmap& pmFolderIconLarge =
                KMimeType::mimeType( "inode/directory" )->pixmap( KIcon::FirstGroup );

            lvItem->setText( FILE_NAME, fileName );
            lvItem->setText( FILE_EXTENSION, QString::null );
            lvItem->setText( FILE_MIMETYPE, QString::null );
            lvItem->setText( FILE_SIZE, QString::null );
            lvItem->setText( FILE_DATE, date.toString( Qt::LocalDate ) );
            lvItem->setText( FILE_TIME, time.toString( Qt::TextDate ) );
            lvItem->setText( FILE_THUMBNAIL, QString::null );
            lvItem->setText( FILE_METADATA, QString::null );
            lvItem->setText( FILE_FULLTEXT, QString::null );
            lvItem->setText( FILE_LANGUAGE, QString::null );

            lvItem->setPixmap( 0, pmFolderIconSmall );

            ivItem->setText( fileName );
            ivItem->setPixmap( pmFolderIconLarge );

            ivThmb->setText( fileName );
            ivThmb->setPixmap( pmFolderIconLarge );

        } else {
            const QPixmap& pmFileIconSmall = mime->pixmap( KIcon::Small );
            const QPixmap& pmFileIconLarge = mime->pixmap( KIcon::FirstGroup );

            QString size = QString::number( info.fileSize );
            addSeparators( size );

            lvItem->setText( FILE_NAME, fileName );
            lvItem->setText( FILE_EXTENSION, ext );
            lvItem->setText( FILE_MIMETYPE, mimeType );
            lvItem->setText( FILE_SIZE, size );
            lvItem->setText( FILE_DATE, date.toString( Qt::LocalDate ) );
            lvItem->setText( FILE_TIME, time.toString( Qt::TextDate ) );
            lvItem->setText( FILE_THUMBNAIL, QString::null );
            lvItem->setText( FILE_METADATA, QString::null );
            lvItem->setText( FILE_FULLTEXT, QString::null );
            lvItem->setText( FILE_LANGUAGE, language );

            lvItem->setPixmap( 0, pmFileIconSmall );

            if ( info.hasThumbnail ) {
                lvItem->setPixmap( FILE_THUMBNAIL, pmCheckIconSmall );
            }
            if ( info.hasMetaData ) {
                lvItem->setPixmap( FILE_METADATA, pmCheckIconSmall );
            }
            if ( info.hasFullText ) {
                lvItem->setPixmap( FILE_FULLTEXT, pmCheckIconSmall );
            }

            ivItem->setText( fileName );
            ivItem->setPixmap( pmFileIconLarge );

            if ( info.thumbnailDataLength == 0 ) {
                ivThmb->setText( fileName );
                ivThmb->setPixmap( pmFileIconLarge );

            } else {
                QPixmap qpthumb( info.thumbnailData );

                ivThmb->setText( fileName );
                ivThmb->setPixmap( qpthumb );
            }
        }
    }
}



QString buildPropertyHTML( const QString &field, const QString &value ) {
    QString p = QString::null;
    p = p + "<tr>";
    p = p + " <td><b>" + field + "</b></td>";
    p = p + " <td>" + value + "</td>";
    p = p + "</tr>";
    return p;
}

void DlgMainWindow::showFileProperties( KatInformation info ) {
    if ( info.fileId == -2L ) { // Catalog

        QString p = QString::null;
        p = p + "<table>";
        p = p + buildPropertyHTML( i18n( "Description" ), m_kat->getDescription() );
        p = p + buildPropertyHTML( i18n( "Notes" ), m_kat->getNotes() );
        p = p + "</table>";

        teProperties->setText( p );

    } else {

        QDateTime mdt;
        mdt.setTime_t( info.modificationDate );
        QDate mdate = mdt.date();
        QTime mtime = mdt.time();

        QString modificationdate = mdate.toString( Qt::LocalDate ) + " " + mtime.toString( Qt::TextDate );

        QString size = QString::number( info.fileSize );
        addSeparators( size );

        QString p = QString::null;
        p = p + "<table>";

        p = p + buildPropertyHTML( "File type", info.fileType );
        p = p + buildPropertyHTML( "File size", size );
        p = p + buildPropertyHTML( "Modification date", modificationdate );

        if ( info.metaData != QString::null ) {
            QString name;
            //FIXME: type not used
            QString type;
            QString value;

            QStringList tokens = QStringList::split( "|", info.metaData, false );

            for ( QStringList::Iterator it = tokens.begin(); it != tokens.end(); it++ ) {
                name = *it;
                it++;
                type = *it;
                it++;
                value = *it;

                p = p + buildPropertyHTML( name, value );
            }
        }

        p = p + "</table>";

        teProperties->setText( p );
    }
}

void DlgMainWindow::populateViews() {
    lvFolders->clear();
    lvFiles->clear();
    ivFiles->clear();
    ivFilesThumbnails->clear();

    KatListViewItem* lvItem = new KatListViewItem( lvFolders );
    lvItem->setText( 0, m_kat->getName() );
    lvItem->setPixmap( 0, pmCatalogIcon );
    lvItem->setOpen( true );

    KatInformation info;
    info.fileType = "KAT_CATALOG";
    info.fileId = -2L;

    lvItem->info = info;

    KatListViewItem* rootItem = new KatListViewItem( lvItem );
    rootItem->setText( 0, m_kat->getPath() );
    rootItem->setPixmap( 0, pmDiscIcon );
    rootItem->setOpen( true );

    info.fileType = "inode/directory";
    info.fileId = -1L;

    rootItem->info = info;

    insertChildrenFolders( -1, rootItem );
}

void DlgMainWindow::populateSearchViews( QValueList<int> results ) {
    QString fileName;
    QString fullName;
    QString ext;
    int pos=0;
    QString language;

    lvResults->clear();

    QValueList<int>::iterator i;
    for ( i = results.begin(); i != results.end(); ++i ) {
        int fileid = *i;

        KatInformation info = dm->readFileInformation( fileid );

        fullName = info.fullName;
        fileName = info.fileName;
        language = info.language;

        KMimeType::Ptr mime = KMimeType::mimeType( info.fileType );
        QString mimeType = info.fileType;

        pos = fileName.findRev( '.', -1 );
        if ( pos == -1 ) {
            ext = QString::null;
        } else {
            ext = fileName.right( fileName.length() - pos - 1 );
            ext = ext.lower();
        }

        QDateTime dt;
        dt.setTime_t( info.modificationDate );
        QDate date = dt.date();
        QTime time = dt.time();

        KatListViewItem* lvItem = new KatListViewItem( lvResults );
        lvItem->info = info;
        lvItem->setDragEnabled( true );
        KatIconViewItem* ivItem = new KatIconViewItem( ivResults );
        ivItem->info = info;
        KatIconViewItem* ivThmb = new KatIconViewItem( ivResultsThumbnails );
        ivThmb->info = info;

        if ( info.fileType == "inode/directory" ) {

            const QPixmap& pmFolderIconSmall =
                KMimeType::mimeType( "inode/directory" )->pixmap( KIcon::Small );
            const QPixmap& pmFolderIconLarge =
                KMimeType::mimeType( "inode/directory" )->pixmap( KIcon::FirstGroup );

            lvItem->setText( FILE_NAME, fileName );
            lvItem->setText( FILE_EXTENSION, QString::null );
            lvItem->setText( FILE_MIMETYPE, QString::null );
            lvItem->setText( FILE_SIZE, QString::null );
            lvItem->setText( FILE_DATE, date.toString( Qt::LocalDate ) );
            lvItem->setText( FILE_TIME, time.toString( Qt::TextDate ) );
            lvItem->setText( FILE_THUMBNAIL, QString::null );
            lvItem->setText( FILE_METADATA, QString::null );
            lvItem->setText( FILE_FULLTEXT, QString::null );
            lvItem->setText( FILE_LANGUAGE, QString::null );

            lvItem->setPixmap( 0, pmFolderIconSmall );

        } else {
            const QPixmap& pmFileIconSmall = mime->pixmap( KIcon::Small );
            const QPixmap& pmFileIconLarge = mime->pixmap( KIcon::FirstGroup );

            QString size = QString::number( info.fileSize );

            QString thousandsSeparator = KGlobal::locale()->thousandsSeparator();
            int i = size.length()-1;
            for( int j=1; i>0; i--, j++ ) {
                if ( j%3 == 0 ) {
                    size.insert(i, thousandsSeparator);
                }
            }

            lvItem->setText( FILE_NAME, fileName );
            lvItem->setText( FILE_EXTENSION, ext );
            lvItem->setText( FILE_MIMETYPE, mimeType );
            lvItem->setText( FILE_SIZE, size );
            lvItem->setText( FILE_DATE, date.toString( Qt::LocalDate ) );
            lvItem->setText( FILE_TIME, time.toString( Qt::TextDate ) );
            lvItem->setText( FILE_THUMBNAIL, QString::null );
            lvItem->setText( FILE_METADATA, QString::null );
            lvItem->setText( FILE_FULLTEXT, QString::null );
            lvItem->setText( FILE_LANGUAGE, language );

            lvItem->setPixmap( 0, pmFileIconSmall );

            if ( info.hasThumbnail ) {
                lvItem->setPixmap( FILE_THUMBNAIL, pmCheckIconSmall );
            }
            if ( info.hasMetaData ) {
                lvItem->setPixmap( FILE_METADATA, pmCheckIconSmall );
            }
            if ( info.hasFullText ) {
                lvItem->setPixmap( FILE_FULLTEXT, pmCheckIconSmall );
            }

            ivItem->setText( fileName );
            ivItem->setPixmap( pmFileIconLarge );

            if ( info.thumbnailDataLength == 0 ) {
                ivThmb->setText( fileName );
                ivThmb->setPixmap( pmFileIconLarge );

            } else {
                QPixmap qpthumb( info.thumbnailData );

                ivThmb->setText( fileName );
                ivThmb->setPixmap( qpthumb );
            }
        }
    }
}

int DlgMainWindow::openFile( KatInformation info ) {
    long fileId = info.fileId;
    QString fullname = info.fullName;

    if ( info.fileType == "inode/directory" ) {

        KatListViewItem* selFolder = static_cast<KatListViewItem*>(lvFolders->selectedItem());
        selFolder->setOpen( true );

        KatListViewItem* child = static_cast<KatListViewItem*>(selFolder->firstChild());
        while ( child != 0 ) {
            if ( child->info.fileId == fileId ) {
                lvFolders->setSelected( child, true );
                child->setOpen( true );
                break;
            }
            child = static_cast<KatListViewItem*>(child->nextSibling());
        }
    } else {
        KURL kurl( info.fullName );
        KMimeType::Ptr mime = KMimeType::mimeType( info.fileType );
        QString mimeType = info.fileType;

        KService::Ptr offer = KServiceTypeProfile::preferredService( mimeType, QString( "Application" ) );

        if ( offer ) {
            struct stat my_stat;
            if ( stat( info.fullName, &my_stat ) ) {
                KMessageBox::error( this, i18n( "The file is currently unavailale. "
                                    "Please mount the filesystem in which the file resides and try again." ) );
            } else {
                KRun::runURL ( kurl, mimeType );
            }
        } else {
            KMessageBox::error( this, i18n( "Unable to open files of type: %1" ).arg( mimeType ) );
        }
    }

    return 0;
}

#include "dlgmainwindow.moc"
