/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _DLGMAINWINDOW_H_
#define _DLGMAINWINDOW_H_

#include <qvariant.h>
#include <kparts/dockmainwindow.h>
#include <kapplication.h>
#include <kdockwidget.h>
#include <kfiledialog.h>
#include <kiconview.h>
#include <klistview.h>
#include <ktextedit.h>
#include <qheader.h>
#include <qiconview.h>
#include <qlayout.h>
#include <qmainwindow.h>
#include <qpixmap.h>
#include <qsplitter.h>
#include <qwidgetstack.h>
#include <qpopupmenu.h>
#include <qvaluelist.h>
#include <kservice.h>
#include <ktrader.h>
#include <dcopclient.h>
#include <dcopobject.h>

#include "katcatalog.h"
#include "katlistviewitem.h"
#include "katiconviewitem.h"
#include "dlgsearch.h"
#include "kat_util.h"
#include "katlistview.h"
#include "katiconview.h"
#include "dlgmainwindowiface.h"

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QPopupMenu;
class QSplitter;
class KListView;
class QListViewItem;
class KAction;
class KToggleAction;
class KRecentFilesAction;
class KConfig;
class KatDatabaseManager;
class KIconLoader;

class DlgMainWindow : public KParts::DockMainWindow, public DlgMainWindowIface {
Q_OBJECT

public:
    DlgMainWindow();
    virtual ~DlgMainWindow();

    void insertChildrenFolders( long, QListViewItem* );
    void insertChildrenFiles( long );
    void showFileProperties( KatInformation );
    void populateViews();
    void populateSearchViews( QValueList<int> );
    void showIcons( KIconLoader*, QIconView* );

    int openFile( KatInformation );
    void createNewCatalog();
    void openCatalog();
    KatCatalog* catalog() const { return m_kat;}
    void setCatalog(KatCatalog*_kat);
    void fileOpen( const KURL &url );

    KatDatabaseManager* databaseManager() const { return dm;}
    LanguageProfileMap languageProfileMap() const { return lp;}

    QString katFilename();

public slots:
    void slotFileNew();
    void slotFileSave();
    void slotFileSaveAs();
    void slotFilePrint();
    void slotFileFind();
    void slotEditUndo();
    void slotEditRedo();
    void slotEditCut();
    void slotEditPaste();
    void slotEditFind();
    void slotFileOpen();
    void slotEditToolbars();
    void slotEditCopy();
    void slotConfigureKeys();

protected slots:
    void selectFolder( QListViewItem* );
    void selectFile( QListViewItem* );
    void selectFileIcon( QIconViewItem* );
    void selectFileThumbnail( QIconViewItem* );
    void contextFolder( KListView*, QListViewItem*, const QPoint & );
    void contextFile( KListView*, QListViewItem*, const QPoint & );
    void contextFileIcon( QIconViewItem*, const QPoint & );
    void contextFileThumbnail( QIconViewItem*, const QPoint & );
    void contextReqFile( QListViewItem*, const QPoint &, int );
    void contextReqFolder( QListViewItem*, const QPoint &, int );
    void doubleClickedFile( QListViewItem* );
    void doubleClickedFileIcon( QIconViewItem* );
    void doubleClickedFileThumbnail( QIconViewItem* );
    void doubleClickedFolder( QListViewItem* );
    void rightButtonFile( QListViewItem*, const QPoint &, int );
    void rightButtonFolder( QListViewItem*, const QPoint &, int );
    void executeFile( QListViewItem* );
    void executeFolder( QListViewItem* );
    void expandFolder( QListViewItem* );
    void slotViewDetailed();
    void slotViewIcons();
    void slotViewThumbnails();
    void slotShowFullText();
    void slotShowWordsList();
    void slotNewToolBarConfig();
    void slotConfigureKat();
    void slotOpenURL( const KURL& );
    void slotSaveProperties();
    void slotOpenWith( int );

protected:
    void initActions();
    void fileSave();
    bool fileNew();
    bool fileSaveAs();
    bool fileOpen();
    void disableAction();

    void initView();
    void initStatusBar();
    void openItemSelected( KatListViewItem* item );
    void openItemSelected( KatIconViewItem* item );

    bool queryClose();

    virtual void saveProperties( KConfig *config );
    virtual void readProperties( KConfig *config );

private:
    QPixmap pmCatalogIcon;

    LanguageProfileMap lp;

    KatDatabaseManager* dm;

    KTrader::OfferList offers;

    QPixmap pmDiscIcon;
    QPixmap pmFileIconSmall;
    QPixmap pmFolderIconSmall;
    QPixmap pmFileIconLarge;
    QPixmap pmFolderIconLarge;
    QPixmap pmCheckIconSmall;

    KAction* fileNewAction;
    KAction* fileOpenAction;
    KAction* fileSaveAction;
    KAction* fileSaveAsAction;
    KAction* filePrintAction;
    KAction* fileExitAction;
    KRecentFilesAction* fileOpenRecent;

    KAction* editUndoAction;
    KAction* editRedoAction;
    KAction* editCutAction;
    KAction* editCopyAction;
    KAction* editPasteAction;
    KAction* editFindAction;

    //icons from toolbar
    KToggleAction* toolbarSearch;
    KAction* toolbarConfigure;
    KToggleAction* toolbarViewDetailed;
    KToggleAction* toolbarViewIcons;
    KToggleAction* toolbarViewThumbnails;

    KDockWidget* dockMain;
    KDockWidget* dockFolders;
    KDockWidget* dockFiles;
    KDockWidget* dockProperties;

    KDockWidget* dockMainSearch;
    KDockWidget* dockSearch;
    KDockWidget* dockResults;

    //KStatusBar* sbStatusBar;

    KatCatalog* m_kat;

    KListView* lvFolders;
    KatListView* lvFiles;
    KatIconView* ivFiles;
    KatIconView* ivFilesThumbnails;
    KTextEdit* teProperties;
    DlgSearch* dsSearch;
    KatListView* lvResults;
    KatIconView* ivResults;
    KatIconView* ivResultsThumbnails;

    QPopupMenu* resultsMenu;
    QPopupMenu* resultsIconsMenu;
    QPopupMenu* resultsThumbnailsMenu;

    QPopupMenu* filesMenu;
    QPopupMenu* filesIconsMenu;
    QPopupMenu* filesThumbnailsMenu;

    KConfig * m_config;
    QString prevFileType;

};

#endif //_DLGMAINWINDOW_H_
