/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kat.h"
#include "dlgnewcatalog.h"
#include "dlgmainwindow.h"
#include "katcatalog.h"
#include "katfulltextjob.h"

#include <qvariant.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <kactionselector.h>
#include <qcheckbox.h>
#include <qslider.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kio/global.h>
#include <kio/jobclasses.h>
#include <kio/previewjob.h>
#include <kio/metainfojob.h>
#include <kurlrequester.h>
#include <klocale.h>
#include <klineedit.h>
#include <qlayout.h>

DlgNewCatalog::DlgNewCatalog( DlgMainWindow* parent, const char* name, bool modal, WFlags fl )
    : KDialogBase( parent, name, modal, i18n( "New Catalog" ) , KDialogBase::Ok|KDialogBase::Cancel|KDialogBase::Help, KDialogBase::Ok, true )
{

    // Get the list of loaded plugins
    mf = parent;

    setSizeGripEnabled( FALSE );
    QWidget *page = new QWidget( this );
    setMainWidget(page);
    QVBoxLayout *DlgNewCatalogLayout = new QVBoxLayout( page, 11, 6, "DlgNewCatalogLayout" );

    tabWidget = new QTabWidget( page, "tabWidget" );

    QWidget *Widget8 = new QWidget( tabWidget, "Widget8" );
    QGridLayout *gridlayout=new QGridLayout( Widget8, 9, 2, KDialog::marginHint(), KDialog::spacingHint());

    tlCatalogName = new QLabel( Widget8, "tlCatalogName" );
    tlCatalogName->setText( i18n( "Catalog name" ) );
    gridlayout->addWidget( tlCatalogName, 0, 0 );

    leCatalogName = new QLineEdit( Widget8, "leCatalogName" );
    gridlayout->addWidget( leCatalogName, 0, 1 );

    tlBaseFolder = new QLabel( Widget8, "tlBaseFolder" );
    gridlayout->addWidget( tlBaseFolder, 1, 0 );
    tlBaseFolder->setText( i18n( "Base folder" ) );

    leBaseFolder = new KURLRequester( Widget8, "leBaseFolder" );
    gridlayout->addWidget( leBaseFolder, 1, 1 );
    leBaseFolder->setMode( KFile::Directory );

    tlDescription = new QLabel( Widget8, "tlDescription" );
    gridlayout->addWidget( tlDescription, 2, 1 );
    tlDescription->setText( i18n( "Description" ) );

    teDescription = new QTextEdit( Widget8, "teDescription" );
    gridlayout->addMultiCellWidget( teDescription, 2,4,1, 1 );
    teDescription->setTabChangesFocus(true);

    tlAuthor = new QLabel( Widget8, "tlAuthor" );
    gridlayout->addWidget( tlAuthor, 5, 0 );
    tlAuthor->setText( i18n( "Author" ) );

    leAuthor = new QLineEdit( Widget8, "leAuthor" );
    gridlayout->addWidget( leAuthor, 5, 1 );

    tlNotes = new QLabel( Widget8, "tlNotes" );
    gridlayout->addWidget( tlNotes, 6, 1 );
    tlNotes->setText( i18n( "Notes" ) );

    teNotes = new QTextEdit( Widget8, "teNotes" );
    gridlayout->addMultiCellWidget( teNotes, 6,8,1, 1 );
    teNotes->setTabChangesFocus(true);
    tabWidget->insertTab( Widget8, i18n( "Catalog" ) );

    QWidget *Widget9 = new QWidget( tabWidget, "Widget9" );
    QVBoxLayout *layout=new QVBoxLayout( Widget9, KDialog::marginHint(), KDialog::spacingHint(), "layout" );
    asDescriptionPlugins = new KActionSelector( Widget9, "asDescriptionPlugins" );
    asDescriptionPlugins->setAvailableLabel( i18n( "&Available:" ) );
    asDescriptionPlugins->setSelectedLabel( i18n( "&Selected:" ) );
    layout->addWidget( asDescriptionPlugins );

    QListBox* lbSelectedDescriptionPlugins = asDescriptionPlugins->selectedListBox();

    QStringList availableMetaInfoPlugins = KIO::MetaInfoJob::availablePlugins();
    for ( QStringList::Iterator it = availableMetaInfoPlugins.begin();
            it != availableMetaInfoPlugins.end(); ++it ) {
        lbSelectedDescriptionPlugins->insertItem( *it );
    }

    tabWidget->insertTab( Widget9, i18n( "Metadata" ) );

    TabPage = new QWidget( tabWidget, "TabPage" );
    layout=new QVBoxLayout( TabPage, KDialog::marginHint(), KDialog::spacingHint(), "layout" );

    asFullTextPlugins = new KActionSelector( TabPage, "asFullTextPlugins" );
    asFullTextPlugins->setAvailableLabel( i18n( "&Available:" ) );
    asFullTextPlugins->setSelectedLabel( i18n( "&Selected:" ) );
    layout->addWidget( asFullTextPlugins );

    QListBox* lbSelectedFullTextPlugins = asFullTextPlugins->selectedListBox();

    QStringList availableFullTextPlugins = KatFullTextJob::availablePlugins();
    for ( QStringList::Iterator it = availableFullTextPlugins.begin();
            it != availableFullTextPlugins.end(); ++it ) {
        lbSelectedFullTextPlugins->insertItem( *it ); //display name
    }

    QHBoxLayout *box = new QHBoxLayout( TabPage );
    cbUseExclusionList = new QCheckBox( TabPage, "cbUseExclusionList" );
    cbUseExclusionList->setText( i18n( "Use exclusion list" ) );
    box->addWidget( cbUseExclusionList );

    bManageExclusionList = new QPushButton( TabPage, "bManageExclusionList" );
    bManageExclusionList->setText( i18n( "Manage exclusion list" ) );
    tabWidget->insertTab( TabPage, i18n( "Fulltext" ) );
    box->addWidget( bManageExclusionList );

    layout->addLayout( box );

    QWidget *TabPage_2 = new QWidget( tabWidget, "TabPage_2" );
    layout=new QVBoxLayout( TabPage_2, KDialog::marginHint(), KDialog::spacingHint(), "layout_2" );

    asThumbnailPlugins = new KActionSelector( TabPage_2, "asThumbnailPlugins" );
    asThumbnailPlugins->setAvailableLabel( i18n( "&Available:" ) );
    asThumbnailPlugins->setSelectedLabel( i18n( "&Selected:" ) );
    layout->addWidget( asThumbnailPlugins );

    QListBox* lbSelectedThumbnailPlugins = asThumbnailPlugins->selectedListBox();

    QStringList availableThumbnailPlugins = KIO::PreviewJob::availablePlugins();
    for ( QStringList::Iterator it = availableThumbnailPlugins.begin();
            it != availableThumbnailPlugins.end(); ++it ) {
        lbSelectedThumbnailPlugins->insertItem( *it  );
    }

    box = new QHBoxLayout( TabPage_2 );

    tlThumbnailSize = new QLabel( TabPage_2, "tlThumbnailSize" );
    box->addWidget( tlThumbnailSize );

    slThumbnailSize = new QSlider( TabPage_2, "slThumbnailSize" );
    slThumbnailSize->setMinValue( 48 );
    slThumbnailSize->setMaxValue( 160 );
    slThumbnailSize->setLineStep( 1 );
    slThumbnailSize->setPageStep( 8 );
    slThumbnailSize->setOrientation( QSlider::Horizontal );
    slThumbnailSize->setTickmarks( QSlider::Below );
    slThumbnailSize->setTickInterval( 8 );
    tlThumbnailSize->setText( i18n( "Thumbnail size" ) );
    QToolTip::add( slThumbnailSize, i18n( "Thumbnail size" ) );
    box->addWidget( slThumbnailSize );

    tlSize = new QLabel( TabPage_2, "tlSize" );
    box->addWidget( tlSize );
    tlSize->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
    tlSize->setText( "48x48" );

    layout->addLayout( box );
    tabWidget->insertTab( TabPage_2, i18n( "Thumbnails" ) );
    DlgNewCatalogLayout->addWidget( tabWidget );

    resize( QSize( 560, 480 ).expandedTo( minimumSizeHint() ) );
    clearWState( WState_Polished );
    connect( bManageExclusionList, SIGNAL( clicked() ),
             this, SLOT( manageExclusionList() ) );
    connect( cbUseExclusionList, SIGNAL( stateChanged(int) ),
             this, SLOT( useExclusionList(int) ) );
    connect( slThumbnailSize, SIGNAL( sliderMoved(int) ),
             this, SLOT( thumbnailSizeSliderMoved(int) ) );
    connect( slThumbnailSize, SIGNAL( valueChanged(int) ),
             this, SLOT( thumbnailSizeValueChanged(int) ) );
    connect( leCatalogName, SIGNAL( textChanged(const QString&) ),
             this, SLOT( changedCatalogname() ) );
    connect( leBaseFolder->lineEdit(), SIGNAL( textChanged(const QString&) ),
             this, SLOT( changedBaseFolder() ) );

    leCatalogName->setFocus();
}

DlgNewCatalog::~DlgNewCatalog() {
}


void DlgNewCatalog::changedCatalogname() {
    if ( !leBaseFolder->lineEdit()->text().isEmpty() && !leCatalogName->text().isEmpty() ) {
        enableButtonOK( true );
    } else {
        enableButtonOK( false );
    }
}

void DlgNewCatalog::slotOk() {
    struct stat my_stat;
    QString basefolder = leBaseFolder->lineEdit()->text();

    // Check if the file exists
    // TODO: Use Qt3 functions (maybe allowing urls beginning with "file:/" )
    int result = stat( basefolder, &my_stat );
    if ( result == -1 ) {
        KMessageBox::error( this, i18n( "The specified folder does not exist or is not accessible" ) );
        return;
    }

    KatCatalog *kat = new KatCatalog();

    kat->setName( leCatalogName->text() );
    kat->setDescription( teDescription->text() );
    kat->setNotes( teNotes->text() );
    kat->setFilename( "" );
    kat->setPath( leBaseFolder->lineEdit()->text() );
    kat->setAuthor( leAuthor->text() );
    kat->setThumbnailSize( slThumbnailSize->value() );
    kat->setCreationDate( time( 0 ) );
    kat->setLastUpdateDate( time( 0 ) );

    mf->setCatalog( kat );

    this->done( QDialog::Accepted );
}



void DlgNewCatalog::useExclusionList( int ) {
}

void DlgNewCatalog::thumbnailSizeSliderMoved( int /*size*/ ) {
}

void DlgNewCatalog::thumbnailSizeValueChanged( int size ) {
    int i;
    int sizes[ 21 ];
    for ( i=0; i < 21; i++ ) {
        sizes[ i ] = i*8;
    }

    for ( i=0; i<20; i++ ) {
        if ( size > sizes[ i ] && size < sizes[ i + 1 ] ) {
            size = sizes[ i ];
            break;
        }
    }

    QString s = QString::number( size ) + "x" + QString::number( size );
    tlSize->setText( s );
    slThumbnailSize->setValue( size );
}

void DlgNewCatalog::changedBaseFolder() {
    if ( !leBaseFolder->lineEdit()->text().isEmpty() && !leCatalogName->text().isEmpty() ) {
        enableButtonOK( true );
    } else {
        enableButtonOK( false );
    }
}

void DlgNewCatalog::manageExclusionList() {
}

#include "dlgnewcatalog.moc"
