/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kstandarddirs.h>
#include <kdebug.h>
#include "kat_export.h"

#include "antiword/antiword.h"
#include "fulltext_doc.h"

extern "C"
{
    KAT_EXPORT FulltextExtractor *new_extractor(const QString& path)
    {
        return new DocExtractor(path);
    }
}

DocExtractor::DocExtractor(const QString &path) :
    m_bDone( false ),
    m_path( path ),
    m_file(path),
    m_stream(&m_file)
{
}

DocExtractor::~DocExtractor()
{
}

void DocExtractor::extract(QString& data)
{
    kdDebug() << "DOCExtractor start" << endl;
    if ( m_bDone ) {
        data = QString::null;
        return;
    }
    m_bDone = true;

    long size;
    FILE *pFile;
    diagram_type *pDiag;

    options_type tOptions;
    vGetOptions( &tOptions );

    if (m_file.open(IO_ReadOnly)) {
        size = m_file.size();
        m_file.close();
    } else {
        data = QString::null;
        return;
    }

    char* text = new char[ size * 2 ];
    strcpy( text, "" );

    pDiag = pCreateDiagram( m_path, text );
    if ( !pDiag ) {
        delete [] text;
        data = QString::null;
        return;
    }

    pFile = fopen( m_path, "rb" );
    if (!pFile ) {
        delete [] text;
        data = QString::null;
        return;
    }
    bWordDecryptor( pFile, size, pDiag );

    QString d( text );
    delete [] text;

    vDestroyDiagram(pDiag);
    fclose(pFile);

    d = d.simplifyWhiteSpace();

    QTextOStream stream( &data );
    stream << "<fulltext>";
    stream << d.local8Bit();
    stream << "</fulltext>";

    kdDebug() << "DOCExtractor end" << endl;
}
