/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kstandarddirs.h>
#include "kat_export.h"

#include "fulltext_html.h"
#include <qregexp.h>
#include <kdebug.h>

extern "C"
{
    KAT_EXPORT FulltextExtractor *new_extractor(const QString& path)
    {
        return new HTMLExtractor(path);
    }
}

HTMLExtractor::HTMLExtractor(const QString &path) :
    m_bDone( false ),
	m_file(path),
	m_stream(&m_file)
{
}

HTMLExtractor::~HTMLExtractor()
{
    m_file.close();
}

void HTMLExtractor::extract(QString& data)
{
	m_open = m_file.open(IO_ReadOnly);	
    kdDebug() << "HTMLExtractor start" << endl;
    if ( m_bDone ) {
        data = QString::null;
        return;
    }
    m_bDone = true;

    if (m_open) {
        QString d = m_stream.read();

        d = d.replace( QRegExp( "<script[^>]*>[^>]*</script>", false ), " " ); // strip scripts
        d = d.replace( QRegExp( "<br>" ), "\n" ); // transform all <br> in \n
        d = d.replace( QRegExp( "<[^>]*>" ), " " ); // strip all HTML tags
        d = d.simplifyWhiteSpace();

        QTextOStream stream( &data );
        stream << "<fulltext>";
        stream << d.local8Bit();
        stream << "</fulltext>";
    }
    m_file.close();
    kdDebug() << "HTMLExtractor end" << endl;
}
