/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kstandarddirs.h>
#include <kdebug.h>

#include "fulltext_text.h"
#include "kat_export.h"

extern "C"
{
    KAT_EXPORT FulltextExtractor *new_extractor(const QString& path)
    {
        return new TextExtractor(path);
    }
}

TextExtractor::TextExtractor(const QString &path) :
    m_file(path),
    m_stream(&m_file),
    m_bDone( false )
{
    m_open = m_file.open(IO_ReadOnly);
}

TextExtractor::~TextExtractor()
{
    m_file.close();
}

void TextExtractor::extract(QString& data)
{
    kdDebug() << "TXTExtractor start" << endl;
    if ( m_bDone ) {
        data = QString::null;
        return;
    }
    m_bDone = true;

    if (m_open) {
        QString d = m_stream.read();

        d = d.simplifyWhiteSpace();

        QTextOStream stream( &data );
        stream << "<fulltext>";
        stream << d.local8Bit();
        stream << "</fulltext>";
    }
    kdDebug() << "TXTExtractor end" << endl;
}

