/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <qdir.h>
#include <iostream>
#include <sstream>
#include <fstream>
#include <qregexp.h>

#include "kat.h"
#include "katcatalog.h"

KatCatalog::KatCatalog() {
    name = "";
    description = "";
    filename = "";
    path = "";
    notes = "";
    author = "";
    thumbnailSize = 0;
    useExclusionList = 0;
    files = 0L;
    folders = 0L;
    words = 0L;
    fullTexts = 0L;
    thumbnails = 0L;
    metaData = 0L;
    extractions = 0L;
    fileSize = 0L;
}

KatCatalog::~KatCatalog() {
    kdDebug()<<" KatCatalog::~KatCatalog() ****************\n";
}

QString KatCatalog::getName() const {
    return name;
}

QString KatCatalog::getDescription() const{
    return description;
}

QString KatCatalog::getNotes() const{
    return notes;
}

QString KatCatalog::getFilename() const{
    return filename;
}

QString KatCatalog::getPath() const{
    return path;
}

QString KatCatalog::getAuthor() const{
    return author;
}

time_t KatCatalog::getCreationDate() const{
    return creationDate;
}

time_t KatCatalog::getLastUpdateDate() const{
    return lastUpdateDate;
}

int KatCatalog::getThumbnailSize() const{
    return thumbnailSize;
}

int KatCatalog::getUseExclusionList() const{
    return useExclusionList;
}

long KatCatalog::getFiles() const{
    return files;
}

long KatCatalog::getFolders() const{
    return folders;
}

long KatCatalog::getWords() const{
    return words;
}

long KatCatalog::getFullTexts() const{
    return fullTexts;
}

long KatCatalog::getMetaData() const{
    return metaData;
}

long KatCatalog::getThumbnails() const{
    return thumbnails;
}

long KatCatalog::getExtractions() const{
    return extractions;
}

KIO::filesize_t KatCatalog::getFileSize() const{
    return fileSize;
}

void KatCatalog::setName( const QString &s ) {
    name = s;
}

void KatCatalog::setDescription( const QString &s ) {
    description = s;
}

void KatCatalog::setNotes( const QString &s ) {
    notes = s;
}

void KatCatalog::setFilename( const QString &s ) {
    filename = s;
}

void KatCatalog::setPath( const QString &s ) {
    path = s;
}

void KatCatalog::setCreationDate( time_t t ) {
    creationDate = t;
}

void KatCatalog::setLastUpdateDate( time_t t ) {
    lastUpdateDate = t;
}

void KatCatalog::setUseExclusionList( int e ) {
    useExclusionList = e;
}

void KatCatalog::setThumbnailSize( int u ) {
    thumbnailSize = u;
}

void KatCatalog::setAuthor( const QString &s ) {
    author = s;
}

void KatCatalog::setFiles( long fi ) {
    files = fi;
}

void KatCatalog::setFolders( long fo ) {
    folders = fo;
}

void KatCatalog::setWords( long w ) {
    words = w;
}

void KatCatalog::setFullTexts( long ft ) {
    fullTexts = ft;
}

void KatCatalog::setMetaData( long md ) {
    metaData = md;
}

void KatCatalog::setThumbnails( long t ) {
    thumbnails = t;
}

void KatCatalog::setExtractions( long e ) {
    extractions = e;
}

void KatCatalog::setFileSize( KIO::filesize_t fs ) {
    fileSize = fs;
}


int KatCatalog::save( QString sourcefile, const QString &destfile ) {
    // TODO: Use Qt3 functions instead of ANSI C

    if ( !destfile.isEmpty() ) {
        filename = destfile;
    }

    if ( sourcefile.isEmpty() ) {
        // Try to move the source file into the destination file
        QString sourcePath = sourcefile.isEmpty() ?QDir::homeDirPath() + QString( "/temp.kat" ) : sourcefile ;

        if ( rename( sourcePath.latin1(), filename.latin1() ) == -1 ) {
            return -1;
        } else {
            return 0;
        }
    } else {

        ifstream in( sourcefile.latin1(), ios::binary );
        ofstream out( filename.latin1(), ios::binary );
        if (!out) {
            return -1;
        }

        char ch;
        while ( in.get(ch) ) {
            out.put( ch );
        }

        if ( !in.eof() ) {
            return -1;
        }

        return 0;
    }

}

int KatCatalog::open( const QString &nfilename ) {
    if ( nfilename.isEmpty() ) {
        filename = nfilename;
    }

    return 0;
}

int KatCatalog::saveCatalogInfo(KatDatabaseManager *_dm) {
    _dm->saveCatalogInformation( *this );

    return 0;
}

int KatCatalog::query( const QString &q ) {
    return 0;
}

#include "katcatalog.moc"
