/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __KATINFOEXTRACTOR_H
#define __KATINFOEXTRACTOR_H

#include <kio/job.h>
#include <kfileitem.h>

#include "katcatalog.h"

class KatDatabaseManager;

class KatInfoExtractor : public KIO::Job {
Q_OBJECT

protected:
    KatInfoExtractor( KatCatalog* catalog, KatDatabaseManager *dm );
    ~KatInfoExtractor() {}

public:
    KIO::filesize_t totalSize() const { return m_totalSize; }
    KIO::filesize_t totalFiles() const { return m_totalFiles; }
    KIO::filesize_t totalSubdirs() const { return m_totalSubdirs; }
    KIO::filesize_t totalPreviews() const { return m_totalPreviews; }
    KIO::filesize_t totalMetaData() const { return m_totalMetaData; }
    KIO::filesize_t totalFullTexts() const { return m_totalFullTexts; }
    long totalExtractions() const { return m_totalExtractions; }
    static KatInfoExtractor* extractInfoJob( KatCatalog* catalog, KatDatabaseManager *dm );
    //static KIO::filesize_t extractInfo( KatCatalog* catalog );

signals:
    void completed();

public slots:
    void slotExtractionAborted();

protected:
    void setSync() { m_bAsync = false; }
    void startNextJob( );

protected slots:
    virtual void slotResult( KIO::Job* job );
    virtual void gotThumbnail( const KFileItem*, const QPixmap& );
    virtual void noThumbnail( const KFileItem* );
    virtual void gotMetaInfo( const KFileItem* );
    virtual void noMetaInfo( const KFileItem* );
    virtual void gotFullText( KIO::Job*, const KFileItem*, const QString& );
    virtual void noFullText( const KFileItem* );
    virtual void resultMetaInfo( KIO::Job* );
    virtual void resultThumbnail( KIO::Job* );
    virtual void resultFullText( KIO::Job* );

private:
    KatCatalog* m_catalog;
    KatDatabaseManager *dm;
    bool m_bAsync;
    long m_jobs;
    KIO::filesize_t m_totalSize;
    KIO::filesize_t m_totalFiles;
    KIO::filesize_t m_totalSubdirs;
    KIO::filesize_t m_fileTableRow;
    KIO::filesize_t m_totalPreviews;
    KIO::filesize_t m_totalMetaData;
    KIO::filesize_t m_totalFullTexts;
    long m_totalExtractions;

protected:
    virtual void virtual_hook( int id, void* data );

private:
    class KatInfoExtractor* d;
};

#endif
