/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KATPREVIEWJOB_H
#define KATPREVIEWJOB_H

#include <kfileitem.h>
#include <kio/job.h>

class QPixmap;

class KatPreviewJob : public KIO::Job {
Q_OBJECT

public:
    KatPreviewJob( const KFileItemList &items, int width, int height,
                   int iconSize, int iconAlpha, bool scale,
                   const QStringList *enabledPlugins, bool deleteItems = false );
    virtual ~KatPreviewJob();

    void removeItem( const KFileItem *item );

    void setIgnoreMaximumSize( bool ignoreSize = true );

    static QStringList availablePlugins();

    static QStringList supportedMimeTypes();

signals:
    void gotPreview( const KFileItem *item, const QPixmap &preview );
    void failed( const KFileItem *item );

protected:
    void getOrCreateThumbnail();
    bool statResultThumbnail();
    void createThumbnail( const QString& );

protected slots:
    virtual void slotResult( KIO::Job *job );
    virtual void previewTimeOut();

private slots:
    void startPreview();
    void slotThumbData( KIO::Job *, const QByteArray & );

private:
    void determineNextFile();
    void emitPreview( const QImage &thumb );
    void emitFailed( const KFileItem *item = 0 );

protected:
    virtual void virtual_hook( int id, void* data );

private:
    struct KatPreviewJobPrivate *d;
};

KatPreviewJob* filePreview( const KFileItemList &items, int width, int height = 0, int iconSize = 0, int iconAlpha = 70, bool scale = true, const QStringList *enabledPlugins = 0 );

KatPreviewJob* filePreview( const KURL::List &items, int width, int height = 0, int iconSize = 0, int iconAlpha = 70, bool scale = true, const QStringList *enabledPlugins = 0 );

#endif
