/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __KATSCANFOLDER_H
#define __KATSCANFOLDER_H

#include <kio/job.h>
#include <kfileitem.h>

#include "katcatalog.h"

class KatDatabaseManager;

class KatScanFolder : public KIO::Job {
Q_OBJECT

protected:
    KatScanFolder( const KURL& directory, KatCatalog* catalog, KatDatabaseManager*_dm );
    ~KatScanFolder() {}

public:
    KIO::filesize_t totalSize() const { return m_totalSize; }
    KIO::filesize_t totalFiles() const { return m_totalFiles; }
    KIO::filesize_t totalSubdirs() const { return m_totalSubdirs; }
    static KatScanFolder* scanFolderJob( const KURL& directory, KatCatalog* catalog, KatDatabaseManager *dm );
    //static KIO::filesize_t scan( const KURL& directory, KatCatalog* catalog );

signals:
    void completed();

public slots:
    void slotScanAborted();

protected:
    void setSync() { m_bAsync = false; }
    void startNextJob( const KURL & url );

protected slots:
    void slotResult( KIO::Job* job );
    void slotEntries( KIO::Job* , const KIO::UDSEntryList& );

private:
    KatCatalog* m_catalog;
    KatDatabaseManager *dm;
    bool m_bAsync;
    KIO::filesize_t m_totalSize;
    KIO::filesize_t m_totalFiles;
    KIO::filesize_t m_totalSubdirs;
    KIO::ListJob* m_lj;

protected:
    virtual void virtual_hook( int id, void* data );

private:
    class KatScanFolder* d;
};

#endif
