/***************************************************************************
 *   Copyright (C) 2005 by Roberto Cappuccio                               *
 *   robertoc@dnet.it                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qcstring.h>
#include <qsocket.h>
#include <qdatetime.h>
#include <qbitarray.h>

#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kinstance.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kurl.h>
#include <ksock.h>

#include "katslave.h"
#include <kat_export.h>

using namespace KIO;

kio_katslaveProtocol::kio_katslaveProtocol(const QCString &pool_socket, const QCString &app_socket)
    : SlaveBase("kio_katslave", pool_socket, app_socket)
{
    kdDebug() << "kio_katslaveProtocol::kio_katslaveProtocol()" << endl;
}

kio_katslaveProtocol::~kio_katslaveProtocol()
{
    kdDebug() << "kio_katslaveProtocol::~kio_katslaveProtocol()" << endl;
}

void kio_katslaveProtocol::get(const KURL& url )
{
    kdDebug() << "kio_katslave::get(const KURL& url)" << endl ;

    kdDebug() << "url.query(): " << url.query() << endl;
    QString remoteServer = url.host();
    int remotePort = url.port();
    kdDebug() << "url.prettyURL(): " << url.prettyURL() << endl;

    infoMessage(i18n("Looking for %1...").arg( remoteServer ) );
    // Send the mimeType as soon as it is known
    mimeType("text/html");
    // Send the data


    QString d = "";

    d = d + "<html>";
    d = d + "<title>Kat - Desktop Search Engine</title>";
    d = d + "<body>";
    d = d + "<p align=\"center\"><img src=\"file:/usr/share/apps/kat/katweb.png\"></p>";
    d = d + "<p align=\"center\">Welcome to <b>Kat</b> - Desktop Search Engine</p>";
    d = d + "<p align=\"center\"></p>";
    d = d + "<form action=\"kat://catalog.kat?search=something\">";
    d = d + "<p align=\"center\">";
    d = d + "<input type=\"text\" name=\"query\">";
    d = d + "</p>";
    d = d + "</form>";
    d = d + "</body>";
    d = d + "</html>";

    //data(QCString(d.local8Bit()));
    data(QCString(d));
    data(QByteArray()); // empty array means we're done sending the data
    finished();
}

void kio_katslaveProtocol::mimetype(const KURL & /*url*/)
{
    mimeType("text/html");
    finished();
}

extern "C"
{
    int KAT_EXPORT kdemain(int argc, char **argv)
    {
        KInstance instance( "kio_katslave" );

        kdDebug(7101) << "*** Starting kio_katslave " << endl;

        if (argc != 4) {
            kdDebug(7101) << "Usage: kio_katslave  protocol domain-socket1 domain-socket2" << endl;
            exit(-1);
        }

        kio_katslaveProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();

        kdDebug(7101) << "*** kio_katslave Done" << endl;
        return 0;
    }
} 
